<?xml version='1.0'?>

<xsl:stylesheet xmlns:xsl      = 'http://www.w3.org/1999/XSL/Transform'
                xmlns:xs       = 'http://www.w3.org/2001/XMLSchema'
                xmlns:soap     = 'http://schemas.xmlsoap.org/wsdl/soap/'
                xmlns:wsdl     = 'http://schemas.xmlsoap.org/wsdl/'
                xmlns:lxslt    = 'http://xml.apache.org/xslt'
                xmlns:java     = 'http://xml.apache.org/xslt/java'
                xmlns:redirect = 'xalan://org.apache.xalan.lib.Redirect'
                xmlns:wf       = 'xalan://com.ibm.workflow.util.fdl.make'
                extension-element-prefixes='redirect'
                version='1.0'>

<xsl:param name='generator'/>
<xsl:param name='ppackage'/>
<xsl:param name='spackage'/>
<xsl:param name='basedir'/>
<xsl:param name='output'/>
<xsl:param name='ddpostfix'/>
<xsl:param name='sns'/>

<xsl:output method='text'/>
<xsl:template match='text()'/>

<xsl:template match="/">
 <xsl:apply-templates select='wsdl:definitions/wsdl:service'/>
</xsl:template>

<xsl:template match='wsdl:service'>
 <xsl:variable name='fileName' select='wf:WSDL.getFileName(string(@name), $basedir, "", "DeploymentScript.bat")'/>
 <redirect:write select='$fileName'>
 <xsl:text>@echo off

: MQSeries Workflow deployment script for WSDL service '</xsl:text>
 <xsl:value-of select='@name'/>
 <xsl:text>'
: This file ("</xsl:text>
 <xsl:variable name='javaFile' select='java:java.io.File.new($fileName)'/>
 <xsl:variable name='absoluteFileName' select='java:getAbsolutePath($javaFile)'/>
 <xsl:value-of select='$absoluteFileName'/>
 <xsl:text>")
: was generated by </xsl:text>
 <xsl:value-of select='$generator'/>
 <xsl:text> on </xsl:text>
 <xsl:value-of select='java:java.util.Date.new()'/>
 <xsl:text>
: from WSDL file "</xsl:text>
 <xsl:value-of select='$output'/>
 <xsl:text>"

@echo Compiling SOAP wrapper and proxies ...</xsl:text>
 <xsl:variable name='spath' select='concat(translate($spackage, ".", "/"), "/")'/>
 <xsl:variable name='ppath' select='concat(translate($ppackage, ".", "/"), "/")'/>
 <xsl:for-each select='//xs:complexType'>
 <xsl:choose>
  <xsl:when test='@name="nullType"'/>
  <xsl:when test='@name="EnquiryResult"'/>
  <xsl:when test='@name="BINARY_ARRAY"'/>
  <xsl:when test='@name="FLOAT_ARRAY"'/>
  <xsl:when test='@name="LONG_ARRAY"'/>
  <xsl:when test='@name="STRING_ARRAY"'/>
  <xsl:otherwise>
  <xsl:variable name='class' select='wf:WSDL.globalName(@name)'/>
  <xsl:text>
javac -classpath .;"%CLASSPATH%" </xsl:text>
  <xsl:value-of select='$spath'/>
  <xsl:value-of select='$class'/>
  <xsl:text>.java</xsl:text>
  </xsl:otherwise>
 </xsl:choose>
 </xsl:for-each>
 <xsl:for-each select='//wsdl:portType'>
  <xsl:text>
javac -classpath .;"%CLASSPATH%" </xsl:text>
  <xsl:value-of select='$ppath'/>
  <xsl:value-of select='wf:WSDL.globalName(@name)'/>
  <xsl:text>Proxy.java</xsl:text>
 </xsl:for-each>
 <xsl:text>
@echo Done.</xsl:text>
 <xsl:for-each select='wsdl:port'>
  <xsl:variable name='binding' select='substring-after(@binding, ":")'/>
  <xsl:variable name='porttype' select='substring-after(//wsdl:binding[@name = $binding]/@type, ":")'/>
  <xsl:text>
@echo Deploying SOAP service '</xsl:text>
  <xsl:value-of select='$porttype'/>
  <xsl:text>' ...
java org.apache.soap.server.ServiceManagerClient </xsl:text>
  <xsl:value-of select='soap:address/@location'/>
  <xsl:text> deploy </xsl:text>
  <xsl:value-of select='$porttype'/>
  <xsl:value-of select='$ddpostfix'/>
  <xsl:text>.xml
@echo Done.</xsl:text>
 </xsl:for-each>
 </redirect:write>
</xsl:template>

</xsl:stylesheet>
