<?xml version='1.0'?>

<xsl:stylesheet xmlns:xsl      = 'http://www.w3.org/1999/XSL/Transform'
                xmlns:xsd      = 'http://www.w3.org/2001/XMLSchema'
                xmlns:soap     = 'http://schemas.xmlsoap.org/wsdl/soap/'
                xmlns:wsdl     = 'http://schemas.xmlsoap.org/wsdl/'
                xmlns:lxslt    = 'http://xml.apache.org/xslt'
                xmlns:java     = 'http://xml.apache.org/xslt/java'
                xmlns:redirect = 'xalan://org.apache.xalan.lib.Redirect'
                xmlns:wf       = 'xalan://com.ibm.workflow.util.wsdl.make'
                extension-element-prefixes='redirect'
                version='1.0'>

<xsl:include href='xsl:/common.xsl'/>

<xsl:param name='generator'/>
<xsl:param name='input'/>
<xsl:param name='output'/>
<xsl:param name='doprocess'/>

<xsl:output method='text' encoding='iso-8859-1' omit-xml-declaration='yes' indent='no'/>

<xsl:template match='text()'/>

<xsl:template match='/'>
 <xsl:apply-templates select='wsdl:definitions/wsdl:service'/>
</xsl:template>

<xsl:template match='wsdl:service'>
 <xsl:variable name='fileName' select='$output'/>
 <xsl:variable name='fileNameProcess' select='concat(substring-before($fileName,".fdl"), "Process.fdl")'/>
 <redirect:open select='$fileName'/>
 <redirect:write select='$fileName'>
  <xsl:call-template name='header'>
   <xsl:with-param name='filename' select='$fileName'/>
  </xsl:call-template>
  <xsl:text>// STRUCTURES</xsl:text>
  <xsl:value-of select='$eol'/>
  <xsl:value-of select='$eol'/>
  <xsl:variable name='ports' select='wsdl:port'/>
  <xsl:variable name='bindings' select='//wsdl:binding[@name = $ports/@binding]'/>
  <xsl:variable name='inout' select='//wsdl:portType[@name = $bindings/@type]/wsdl:operation/*[self::wsdl:input or self::wsdl:output]'/>
  <xsl:for-each select='wf:FDL.getStructureFdl(string(../@targetNamespace), $inout)'>
   <xsl:value-of select='.'/>
   <xsl:value-of select='$eol'/>
  </xsl:for-each>
  <xsl:text>// SOAP UPES</xsl:text>
  <xsl:value-of select='$eol'/>
  <xsl:value-of select='$eol'/>
  <xsl:text>SERVER '</xsl:text>
  <xsl:value-of select='wf:FDL.checkedSubst("UpesName")'/>
  <xsl:text>'</xsl:text>
  <xsl:value-of select='$eol'/>
  <xsl:text>  DESCRIPTION 'UPES for invocation of Web Services via SOAP'</xsl:text>
  <xsl:value-of select='$eol'/>
  <xsl:text>  RELATED_GROUP '</xsl:text>
  <xsl:value-of select='wf:FDL.checkedSubst("UpesGroup")'/>
  <xsl:text>'</xsl:text>
  <xsl:value-of select='$eol'/>
  <xsl:text>  RELATED_SYSTEM '</xsl:text>
  <xsl:value-of select='wf:FDL.checkedSubst("UpesSystem")'/>
  <xsl:text>'</xsl:text>
  <xsl:value-of select='$eol'/>
  <xsl:text>  TYPE USER_DEFINED_PROGRAM_EXECUTION_SERVER</xsl:text>
  <xsl:value-of select='$eol'/>
  <xsl:text>  MQSYSTEM MQSERIES</xsl:text>
  <xsl:value-of select='$eol'/>
  <xsl:text>  MESSAGE_FORMAT XML</xsl:text>
  <xsl:value-of select='$eol'/>
  <xsl:text>  PHYSICAL_QUEUE_NAME '</xsl:text>
  <xsl:value-of select='wf:FDL.checkedSubst("UpesQueueName")'/>
  <xsl:text>'</xsl:text>
  <xsl:value-of select='$eol'/>
  <xsl:text>  QUEUE_MANAGER_NAME '</xsl:text>
  <xsl:value-of select='wf:FDL.checkedSubst("UpesQueueManagerName")'/>
  <xsl:text>'</xsl:text>
  <xsl:value-of select='$eol'/>
  <xsl:text>  VERSION </xsl:text>
  <xsl:value-of select='$version'/>
  <xsl:value-of select='$eol'/>
  <xsl:text>  RELEASE </xsl:text>
  <xsl:value-of select='$release'/>
  <xsl:value-of select='$eol'/>
  <xsl:text>  LEVEL </xsl:text>
  <xsl:value-of select='$level'/>
  <xsl:value-of select='$eol'/>
  <xsl:text>END '</xsl:text>
  <xsl:value-of select='wf:FDL.checkedSubst("UpesName")'/>
  <xsl:text>'</xsl:text>
  <xsl:value-of select='$eol'/>
  <xsl:value-of select='$eol'/>
  <xsl:text>// PROGRAMS</xsl:text>
  <xsl:value-of select='$eol'/>
  <xsl:value-of select='$eol'/>
  <xsl:if test='$doprocess'>
   <redirect:open select='$fileNameProcess'/>
   <redirect:write select='$fileNameProcess'>
    <xsl:call-template name='header'>
     <xsl:with-param name='filename' select='$fileNameProcess'/>
    </xsl:call-template>
    <xsl:text>// PROCESSES</xsl:text>
    <xsl:value-of select='$eol'/>
    <xsl:value-of select='$eol'/>
   </redirect:write>
  </xsl:if>
  <xsl:for-each select='//wsdl:portType[@name = $bindings/@type]'>
   <xsl:for-each select='wsdl:operation'>
    <xsl:choose>
     <xsl:when test='../@name = @name'>
      <xsl:variable name='name' select='wf:FDL.checkedSubst(string(@name))'/>
      <xsl:call-template name='program'>
       <xsl:with-param name='program' select='wf:FDL.checkedSubst($name)'/>
      </xsl:call-template>
      <xsl:if test='$doprocess'>
       <redirect:write select='$fileNameProcess'>
        <xsl:call-template name='process'>
         <xsl:with-param name='process' select='wf:FDL.checkedSubst($name)'/>
        </xsl:call-template>
       </redirect:write>
      </xsl:if>
     </xsl:when>
     <xsl:otherwise>
      <xsl:variable name='name' select='concat(wf:FDL.checkedSubst(string(../@name)), "_", @name)'/>
      <xsl:call-template name='program'>
       <xsl:with-param name='program' select='wf:FDL.checkedSubst($name)'/>
      </xsl:call-template>
      <xsl:if test='$doprocess'>
       <redirect:write select='$fileNameProcess'>
        <xsl:call-template name='process'>
         <xsl:with-param name='process' select='wf:FDL.checkedSubst($name)'/>
        </xsl:call-template>
       </redirect:write>
      </xsl:if>
     </xsl:otherwise>
    </xsl:choose>
   </xsl:for-each>
  </xsl:for-each>
  <xsl:if test='$doprocess'>
   <redirect:close select='$fileNameProcess'/>
  </xsl:if>
  </redirect:write>
 <redirect:close select='$fileName'/>
</xsl:template>

<xsl:template name='header'>
 <xsl:param name='filename'/>
 <xsl:text>// MQSeries Workflow FDL for WSDL service '</xsl:text>
 <xsl:value-of select='@name'/>
 <xsl:text>'</xsl:text>
 <xsl:value-of select='$eol'/>
 <xsl:text>// This file ("</xsl:text>
 <xsl:variable name='javaFile' select='java:java.io.File.new($filename)'/>
 <xsl:variable name='absoluteFileName' select='java:getAbsolutePath($javaFile)'/>
 <xsl:value-of select='$absoluteFileName'/>
 <xsl:text>")</xsl:text>
 <xsl:value-of select='$eol'/>
 <xsl:text>// was generated by </xsl:text>
 <xsl:value-of select='$generator'/>
 <xsl:text> on </xsl:text>
 <xsl:value-of select='java:java.util.Date.new()'/>
 <xsl:value-of select='$eol'/>
 <xsl:text>// from WSDL file "</xsl:text>
 <xsl:value-of select='$input'/>
 <xsl:text>"</xsl:text>
 <xsl:value-of select='$eol'/>
 <xsl:value-of select='$eol'/>
 <xsl:value-of select='$codepage'/>
 <xsl:value-of select='$eol'/>
 <xsl:value-of select='$fmrelease'/>
 <xsl:value-of select='$eol'/>
 <xsl:value-of select='$eol'/>
</xsl:template>

<xsl:template name='opsig'>
 <xsl:text>'('</xsl:text>
 <xsl:choose>
  <xsl:when test='//wsdl:message[@name = current()/wsdl:input/@message]/wsdl:part'>
   <xsl:value-of select='wsdl:input/@message'/>
  </xsl:when>
  <xsl:otherwise>
   <xsl:value-of select='"Default Data Structure"'/>
  </xsl:otherwise>
 </xsl:choose>
 <xsl:text>', '</xsl:text>
 <xsl:choose>
  <xsl:when test='//wsdl:message[@name = current()/wsdl:output/@message]/wsdl:part'>
   <xsl:value-of select='wsdl:output/@message'/>
  </xsl:when>
  <xsl:otherwise>
   <xsl:value-of select='"Default Data Structure"'/>
  </xsl:otherwise>
 </xsl:choose>
 <xsl:text>')</xsl:text>
 <xsl:value-of select='$eol'/>
</xsl:template>

<xsl:template name='program'>
 <xsl:param name='program'/>
 <xsl:text>PROGRAM '</xsl:text>
 <xsl:value-of select='$program'/>
 <xsl:call-template name='opsig'/>
 <xsl:text>  NO STRUCTURES_FROM_ACTIVITY</xsl:text>
 <xsl:value-of select='$eol'/>
 <xsl:text>  UNATTENDED</xsl:text>
 <xsl:value-of select='$eol'/>
 <xsl:text>  WINNT EXE PATH_AND_FILENAME 'NeverInvoked.exe'</xsl:text>
 <xsl:value-of select='$eol'/>
 <xsl:text>    PARAMETER '</xsl:text>
 <xsl:value-of select='wf:FDL.getParameter($program, .)'/>
 <xsl:text>'</xsl:text>
 <xsl:value-of select='$eol'/>
 <xsl:text>    INHERIT ENVIRONMENT</xsl:text>
 <xsl:value-of select='$eol'/>
 <xsl:text>    STYLE VISIBLE</xsl:text>
 <xsl:value-of select='$eol'/>
 <xsl:text>    START FOREGROUND</xsl:text>
 <xsl:value-of select='$eol'/>
 <xsl:text>END '</xsl:text>
 <xsl:value-of select='$program'/>
 <xsl:text>'</xsl:text>
 <xsl:value-of select='$eol'/>
 <xsl:value-of select='$eol'/>
</xsl:template>

<xsl:template name='process'>
 <xsl:param name='process'/>
 <xsl:text>PROCESS '</xsl:text>
 <xsl:value-of select='$process'/>
 <xsl:call-template name='opsig'/>
 <xsl:text>  DO NOT PROMPT_AT_PROCESS_START</xsl:text>
 <xsl:value-of select='$eol'/>
 <xsl:text>  FULL AUDIT</xsl:text>
 <xsl:value-of select='$eol'/>
 <xsl:text>  KEEP_PROCESSES FOREVER</xsl:text>
 <xsl:value-of select='$eol'/>
 <xsl:text>  WINDOW VIEW_REFERENCE_POINT XPOS 0 YPOS 0</xsl:text>
 <xsl:value-of select='$eol'/>
 <xsl:text>  WINDOW SHOW ALL CONNECTORS</xsl:text>
 <xsl:value-of select='$eol'/>
 <xsl:text>  WINDOW SHOW ALL CONDITIONS</xsl:text>
 <xsl:value-of select='$eol'/>
 <xsl:if test='//wsdl:message[@name = current()/wsdl:input/@message]/wsdl:part'>
  <xsl:text>  SOURCE 1 XPOS -500 YPOS 0</xsl:text>
  <xsl:value-of select='$eol'/>
 </xsl:if>
 <xsl:if test='//wsdl:message[@name = current()/wsdl:output/@message]/wsdl:part'>
  <xsl:text>  SINK   1 XPOS  500 YPOS 0</xsl:text>
  <xsl:value-of select='$eol'/>
 </xsl:if>
 <xsl:text>  PROGRAM_ACTIVITY '</xsl:text>
 <xsl:value-of select='$process'/>
 <xsl:call-template name='opsig'/>
 <xsl:text>    START AUTOMATIC WHEN AT_LEAST_ONE CONNECTOR TRUE</xsl:text>
 <xsl:value-of select='$eol'/>
 <xsl:text>    EXIT AUTOMATIC</xsl:text>
 <xsl:value-of select='$eol'/>
 <xsl:text>    LAYOUT        XPOS 0 YPOS 0</xsl:text>
 <xsl:value-of select='$eol'/>
 <xsl:text>    NAME_POSITION XPOS 0 YPOS -75</xsl:text>
 <xsl:value-of select='$eol'/>
 <xsl:text>    PRIORITY DEFINED_IN INPUT_CONTAINER</xsl:text>
 <xsl:value-of select='$eol'/>
 <xsl:text>    NO CHECKOUT_POSSIBLE</xsl:text>
 <xsl:value-of select='$eol'/>
 <xsl:text>    DONE_BY PROCESS_STARTER</xsl:text>
 <xsl:value-of select='$eol'/>
 <xsl:text>    PROGRAM '</xsl:text>
 <xsl:value-of select='$process'/>
 <xsl:text>'</xsl:text>
 <xsl:value-of select='$eol'/>
 <xsl:text>    PROGRAM_EXECUTION_UNIT '</xsl:text>
 <xsl:value-of select='wf:FDL.checkedSubst("UpesName")'/>
 <xsl:text>.</xsl:text>
 <xsl:value-of select='wf:FDL.checkedSubst("UpesSystem")'/>
 <xsl:text>.</xsl:text>
 <xsl:value-of select='wf:FDL.checkedSubst("UpesGroup")'/>
 <xsl:text>'</xsl:text>
 <xsl:value-of select='$eol'/>
 <xsl:text>    SYNCHRONIZATION </xsl:text>
 <xsl:choose>
  <xsl:when test='wsdl:output'>
   <xsl:text>NESTED</xsl:text>
  </xsl:when>
  <xsl:otherwise>
   <xsl:text>CHAINED</xsl:text>
  </xsl:otherwise>
 </xsl:choose>
 <xsl:value-of select='$eol'/>
 <xsl:text>  END '</xsl:text>
 <xsl:value-of select='$process'/>
 <xsl:text>'</xsl:text>
 <xsl:value-of select='$eol'/>
 <xsl:if test='//wsdl:message[@name = current()/wsdl:input/@message]/wsdl:part'>
  <xsl:text>  DATA</xsl:text>
  <xsl:value-of select='$eol'/>
  <xsl:text>    FROM SOURCE 1 TO '</xsl:text>
  <xsl:value-of select='$process'/>
  <xsl:text>'</xsl:text>
  <xsl:value-of select='$eol'/>
  <xsl:text>    MAP '_STRUCT' TO '_STRUCT'</xsl:text>
  <xsl:value-of select='$eol'/>
 </xsl:if>
 <xsl:if test='//wsdl:message[@name = current()/wsdl:output/@message]/wsdl:part'>
  <xsl:text>  DATA</xsl:text>
  <xsl:value-of select='$eol'/>
  <xsl:text>    FROM '</xsl:text>
  <xsl:value-of select='$process'/>
  <xsl:text>' TO SINK 1</xsl:text>
  <xsl:value-of select='$eol'/>
  <xsl:text>    MAP '_STRUCT' TO '_STRUCT'</xsl:text>
  <xsl:value-of select='$eol'/>
 </xsl:if>
 <xsl:text>END '</xsl:text>
 <xsl:value-of select='$process'/>
 <xsl:text>'</xsl:text>
 <xsl:value-of select='$eol'/>
 <xsl:value-of select='$eol'/>
</xsl:template>

</xsl:stylesheet>
