<?xml version="1.0" ?>


<!-- ********************************************************* -->
<!-- *                                                       * -->
<!-- ********************************************************* -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:wsfl="http://schemas.xmlsoap.org/wsfl"
                xmlns:wsdl="http://schemas.xmlsoap.org/wsdl"
                xmlns:wf="http://www.ibm.com/workflow/2001"
                xmlns:wfpi="http://www.ibm.com/workflow/2001/programInvocation"
                xmlns:wfpres="http://www.ibm.com/workflow/2001/presentation"
                xmlns:wfsq="http://www.ibm.com/workflow/2001/staffQuery"
                version="1.0">

<xsl:import href="xsl:/common.xsl"/>


<!-- ********************************************************* -->
<!-- * PROCESS                                               * -->
<!-- *   Param: ini - initial Indent (required for layout)   * -->
<!-- ********************************************************* -->

<xsl:template match="wsfl:flowModel">
  <xsl:param name="ini"/>
  <xsl:param name="procOption"/>
  <xsl:call-template name="heading">
    <xsl:with-param name="str" select="concat('PROCESS ',@name)"/>
  </xsl:call-template>
  <xsl:if test="$procOption='Create'">
    <xsl:text>CREATE&#x20;</xsl:text>
  </xsl:if>
  <xsl:if test="$procOption='Update'">
    <xsl:text>UPDATE&#x20;</xsl:text>
  </xsl:if>
  <xsl:if test="$procOption='Replace'">
    <xsl:text>REPLACE&#x20;</xsl:text>
  </xsl:if>
  <xsl:text>PROCESS </xsl:text>
  <xsl:call-template name="singlequote">
    <xsl:with-param name="string" select="@name"/>
  </xsl:call-template>
  <xsl:call-template name="my-signature">
    <xsl:with-param name="input"
                    select="wsfl:flowSource/wsdl:output/@message"/>
    <xsl:with-param name="output" select="wsfl:flowSink/wsdl:input/@message"/>
  </xsl:call-template>
  <xsl:value-of select='$eol'/>
  <xsl:apply-templates select="//wf:flowModel">
    <xsl:with-param name="ini" select="concat($ini, $indent)"/>
  </xsl:apply-templates>
  <xsl:apply-templates select="//wf:flowSource"/>
  <xsl:apply-templates select="//wf:flowSink"/>
  <xsl:apply-templates select="wsfl:activity">
    <xsl:with-param name="ini" select="$indent"/>
  </xsl:apply-templates>
  <xsl:apply-templates select="wsfl:controlLink">
    <xsl:with-param name="ini" select="$indent"/>
  </xsl:apply-templates>
  <xsl:apply-templates select="wsfl:dataLink">
    <xsl:with-param name="ini" select="$indent"/>
  </xsl:apply-templates>
  <xsl:text>END </xsl:text>
  <xsl:call-template name="singlequote">
    <xsl:with-param name="string" select="@name"/>
  </xsl:call-template>
  <xsl:value-of select='$eol'/>
  <xsl:value-of select='$eol'/>
</xsl:template>


<!-- ********************************************************* -->
<!-- * PROCESS PROPERTIES                                    * -->
<!-- ********************************************************* -->

<xsl:template name="processProperties" match="wf:flowModel">
  <xsl:param name="ini"/>
  <xsl:apply-templates select="wf:description">
    <xsl:with-param name="ini" select="$ini"/>
  </xsl:apply-templates>
  <xsl:value-of select='$eol'/>
  <xsl:if test="@category">
    <xsl:value-of select="$ini"/>
    <xsl:text>CATEGORY </xsl:text>
    <xsl:call-template name="singlequote">
      <xsl:with-param name="string" select="@category"/>
    </xsl:call-template>
    <xsl:value-of select='$eol'/>
  </xsl:if>
  <xsl:if test="@validFrom">
    <xsl:value-of select="$ini"/>
      <xsl:text>VALID_FROM </xsl:text>
    <xsl:call-template name="singlequote">
      <xsl:with-param name="string" select="@validFrom"/>
    </xsl:call-template>
    <xsl:value-of select='$eol'/>
  </xsl:if>
  <xsl:if test="@promptAtProcessStart">
    <xsl:value-of select="$ini"/>
    <xsl:if test="@promptAtProcessStart='no'">
      <xsl:text>DO NOT </xsl:text>
    </xsl:if>
    <xsl:text>PROMPT_AT_PROCESS_START </xsl:text>
    <xsl:value-of select='$eol'/>
  </xsl:if>
  <xsl:if test="@rcCreation">
    <xsl:value-of select="$ini"/>
    <xsl:text>RC_CREATION </xsl:text>
    <xsl:variable name="rcc">
      <xsl:call-template name="toUpperCase">
        <xsl:with-param name="string" select="string(@rcCreation)"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:value-of select="$rcc"/>
    <xsl:value-of select='$eol'/>
  </xsl:if>
</xsl:template>

<xsl:template match="wf:flowSource">
  <xsl:value-of select="$indent"/>
  <xsl:text>SOURCE </xsl:text>
  <xsl:value-of select="@name"/>
  <xsl:if test="wf:presentation">
    <xsl:call-template name="coords">
      <xsl:with-param name="node" select="wf:presentation/wfpres:symbol"/>
    </xsl:call-template>
  </xsl:if>
  <xsl:value-of select='$eol'/>
</xsl:template>

<xsl:template match="wf:flowSink">
  <xsl:value-of select="$indent"/>
  <xsl:text>SINK </xsl:text>
  <xsl:value-of select="@name"/>
  <xsl:if test="wf:presentation">
    <xsl:call-template name="coords">
      <xsl:with-param name="node" select="wf:presentation/wfpres:symbol"/>
    </xsl:call-template>
  </xsl:if>
  <xsl:value-of select='$eol'/>
</xsl:template>

<!-- ********************************************************* -->
<!-- * ACTIVITY                                              * -->
<!-- ********************************************************* -->

<xsl:template match="wsfl:activity">
  <xsl:param name="ini"/>
  <xsl:value-of select="$ini"/>
  <xsl:text>PROGRAM_ACTIVITY </xsl:text>
  <xsl:call-template name="singlequote">
    <xsl:with-param name="string" select="@name"/>
  </xsl:call-template>
  <xsl:call-template name="my-signature">
    <xsl:with-param name="input" select="wsdl:input/@message"/>
    <xsl:with-param name="output" select="wsdl:output/@message"/>
  </xsl:call-template>
  <xsl:value-of select='$eol'/>
  <xsl:variable name="my-name" select="./@name"/>
  <xsl:variable name="wf-activity"
          select="//wf:deployment//wsfl:activity[@name=$my-name]/wf:activity"/>
  <xsl:call-template name="activity-settings">
    <xsl:with-param name="ini" select="$ini"/>
    <xsl:with-param name="wf-activity" select="$wf-activity"/>
  </xsl:call-template>
  <xsl:apply-templates select="wsfl:implement/wsfl:internal"
                       mode="emit-program-ref">
    <xsl:with-param name="ini" select="$ini"/>
    <xsl:with-param name="wf-activity" select="$wf-activity"/>
  </xsl:apply-templates>
  <xsl:value-of select="$ini"/>
  <xsl:text>END </xsl:text>
  <xsl:call-template name="singlequote">
    <xsl:with-param name="string" select="@name"/>
  </xsl:call-template>
  <xsl:value-of select='$eol'/>
</xsl:template>

<xsl:template name="activity-settings">
  <xsl:param name="ini"/>
  <xsl:param name="wf-activity"/>
  <xsl:apply-templates select="wsfl:join">
    <xsl:with-param name="ini" select="$ini"/>
    <xsl:with-param name="wf-activity" select="$wf-activity"/>
  </xsl:apply-templates>
  <xsl:if test="@exitCondition">
    <xsl:value-of select="concat($ini, $indent, 'EXIT ')"/>
    <xsl:if test="string($wf-activity/@exitType)='manual'">
      <xsl:text>MANUAL </xsl:text>
    </xsl:if>
    <xsl:if test="string($wf-activity/@exitType)='automatic'">
      <xsl:text>AUTOMATIC </xsl:text>
    </xsl:if>
    <xsl:value-of select='$eol'/>
  </xsl:if>
  <xsl:apply-templates select="$wf-activity/wf:presentation/wfpres:symbol">
    <xsl:with-param name="ini" select="$ini"/>
  </xsl:apply-templates>
  <xsl:apply-templates select="$wf-activity/wf:presentation/wfpres:name">
    <xsl:with-param name="ini" select="$ini"/>
  </xsl:apply-templates>
  <xsl:apply-templates select="$wf-activity/wf:priority">
    <xsl:with-param name="ini" select="$ini"/>
  </xsl:apply-templates>
  <xsl:apply-templates select="$wf-activity/wf:staff">
    <xsl:with-param name="ini" select="$ini"/>
  </xsl:apply-templates>
</xsl:template>

<xsl:template match="wsfl:join">
  <xsl:param name="ini"/>
  <xsl:param name="wf-activity"/>
  <xsl:value-of select="concat($ini, $indent, 'START ')"/>
  <xsl:if test="string($wf-activity/wf:join/@startType)='manual'">
    <xsl:text>MANUAL WHEN </xsl:text>
  </xsl:if>
  <xsl:if test="string($wf-activity/wf:join/@startType)='automatic'">
    <xsl:text>AUTOMATIC WHEN </xsl:text>
  </xsl:if>
  <xsl:if test="string(@condition)='all'">
    <xsl:text>ALL CONNECTORS TRUE</xsl:text>
  </xsl:if>
  <xsl:if test="string(@condition)='one'">
    <xsl:text>AT_LEAST_ONE CONNECTOR TRUE</xsl:text>
  </xsl:if>
  <xsl:value-of select='$eol'/>
</xsl:template>

<xsl:template match="wfpres:symbol">
  <xsl:param name="ini"/>
  <xsl:value-of select="concat($ini, $indent, 'LAYOUT')"/>
  <xsl:call-template name="coords">
    <xsl:with-param name="node" select="."/>
  </xsl:call-template>
  <xsl:value-of select='$eol'/>
</xsl:template>

<xsl:template match="wfpres:name">
  <xsl:param name="ini"/>
  <xsl:value-of select="concat($ini, $indent, 'NAME_POSITION')"/>
  <xsl:call-template name="coords">
    <xsl:with-param name="node" select="."/>
  </xsl:call-template>
  <xsl:value-of select='$eol'/>
</xsl:template>

<xsl:template match="wf:staff">
  <xsl:param name="ini"/>
  <xsl:if test="wfsq:processStarter">
    <xsl:value-of select="concat($ini, $indent, 'DONE_BY PROCESS_STARTER')"/>
    <xsl:value-of select='$eol'/>
  </xsl:if>
</xsl:template>

<xsl:template match="wf:priority">
  <xsl:param name="ini"/>
  <xsl:value-of select="concat($ini, $indent, 'PRIORITY ')"/>
  <xsl:if test="@definedInInputContainer='yes'">
    <xsl:text>DEFINED_IN INPUT_CONTAINER</xsl:text>
  </xsl:if>
  <xsl:value-of select='$eol'/>
</xsl:template>

<xsl:template match="wsfl:internal" mode="emit-program-ref">
  <xsl:param name="ini"/>
  <xsl:param name="wf-activity"/>
  <xsl:variable name="my-sp">
    <xsl:call-template name="removePrefix">
      <xsl:with-param name="qname" select="@serviceProvider"/>
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="my-pt">
    <xsl:call-template name="removePrefix">
      <xsl:with-param name="qname" select="@portType"/>
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="my-opn">
    <xsl:call-template name="removePrefix">
      <xsl:with-param name="qname" select="@operation"/>
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="service-provider"
                select="//wf:deployment//wsfl:serviceProvider[@name=$my-sp]"/>
  <xsl:variable name="source" select="$service-provider//wf:locator/@source"/>
  <xsl:variable name="int-prog-doc"
                select="document('xsl:./internal.xsl')"/>
  <xsl:variable name="exe-prog-url">
    <xsl:choose>
      <xsl:when test="string($source)='EXE'">
        <xsl:value-of select="$service-provider//wf:locator/@url"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="'xsl:./internal.xsl'"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <xsl:variable name="exe-prog-doc" select="document($exe-prog-url, /)"/>
  <xsl:variable name="service-name">
    <xsl:call-template name="removePrefix">
      <xsl:with-param name="qname"
                      select="$service-provider//wf:locator/@service"/>
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="service"
        select="$exe-prog-doc//wsdl:service[@name=$service-name]"/>
  <xsl:variable name="binding-name">
    <xsl:call-template name="removePrefix">
      <xsl:with-param name="qname"
                      select="$service/wsdl:port[@name=$my-pt]/@binding"/>
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="binding"
                    select="$exe-prog-doc//wsdl:binding[@name=$binding-name]"/>
  <xsl:variable name="w-b-opn"
                select="$int-prog-doc//wsdl:binding/wsdl:operation"/>
  <xsl:variable name="e-b-opn"
                select="$binding/wsdl:operation[@name=$my-opn]"/>
  <xsl:value-of select="concat($ini, $indent, 'PROGRAM ')"/>
  <xsl:call-template name="singlequote">
    <xsl:with-param name="string" select="$my-opn"/>
  </xsl:call-template>
  <xsl:value-of select='$eol'/>
  <xsl:if test="$wf-activity/wf:programExecutionUnit">
    <xsl:value-of select="concat($ini, $indent, 'PROGRAM_EXECUTION_UNIT ')"/>
    <xsl:call-template name="singlequote">
      <xsl:with-param name="string"
                      select="$wf-activity/wf:programExecutionUnit/@name"/>
    </xsl:call-template>
    <xsl:value-of select='$eol'/>
  </xsl:if>
  <xsl:choose>
    <xsl:when test="string($source)='WSDL'">
      <xsl:apply-templates select="$w-b-opn/wfpi:operation"
                           mode="emit-program-ref">
  <xsl:with-param name="ini" select="$indent"/>
      </xsl:apply-templates>
    </xsl:when>
    <xsl:otherwise>
      <xsl:apply-templates select="$e-b-opn/wfpi:operation"
                           mode="emit-program-ref">
  <xsl:with-param name="ini" select="$indent"/>
      </xsl:apply-templates>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template match="wfpi:operation" mode="emit-program-ref">
  <xsl:param name="ini"/>
  <xsl:if test="@synchronization">
    <xsl:value-of select="concat($ini, $indent, 'SYNCHRONIZATION ')"/>
    <xsl:variable name="sync-val">
      <xsl:call-template name="toUpperCase">
  <xsl:with-param name="string" select="@synchronization"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:value-of select="$sync-val"/>
    <xsl:value-of select='$eol'/>
  </xsl:if>
</xsl:template>

<!-- ********************************************************* -->
<!-- * DATALINK                                              * -->
<!-- ********************************************************* -->

<xsl:template match="wsfl:dataLink">
  <xsl:param name="ini"/>
  <xsl:value-of select="$ini"/>
  <xsl:text>DATA </xsl:text>
  <xsl:value-of select='$eol'/>
  <xsl:variable name="my-name" select="./@name"/>
  <xsl:call-template name="data-link-settings">
    <xsl:with-param name="ini" select="$ini"/>
    <xsl:with-param name="wf-data-link"
            select="//wf:deployment//dataLink[@name=$my-name]/wf:dataLink"/>
  </xsl:call-template>
  <xsl:choose>
    <xsl:when test="wsfl:map">
      <xsl:apply-templates select="wsfl:map" mode="emit-data-link-map">
  <xsl:with-param name="ini" select="$ini"/>
      </xsl:apply-templates>
    </xsl:when>
    <xsl:otherwise>
      <xsl:call-template name="emit-default-data-link-map">
  <xsl:with-param name="ini" select="$ini"/>
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name="data-link-settings">
  <xsl:param name="ini"/>
  <xsl:param name="wf-data-link"/>
  <xsl:if test="@description">
    <xsl:value-of select="concat($ini, $indent)"/>
    <xsl:text>DESCRIPTION </xsl:text>
    <xsl:value-of select="@description"/>
    <xsl:value-of select='$eol'/>
  </xsl:if>
  <xsl:variable name="my-source">
    <xsl:call-template name="removePrefix">
      <xsl:with-param name="qname" select="./@source"/>
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="wf-flow-source-node"
    select="//wf:deployment//wsfl:flowSource[@name=$my-source]/wf:flowSource"/>
  <xsl:choose>
    <xsl:when test="$wf-flow-source-node">
      <xsl:value-of select="concat($ini, $indent)"/>
      <xsl:text>FROM SOURCE </xsl:text>
      <xsl:value-of select="$wf-flow-source-node/@name"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="concat($ini, $indent)"/>
      <xsl:text>FROM </xsl:text>
      <xsl:call-template name="singlequote">
  <xsl:with-param name="string" select="$my-source"/>
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>
  <xsl:variable name="my-target">
    <xsl:call-template name="removePrefix">
      <xsl:with-param name="qname" select="./@target"/>
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="wf-flow-sink-node"
        select="//wf:deployment//wsfl:flowSink[@name=$my-target]/wf:flowSink"/>
  <xsl:choose>
    <xsl:when test="$wf-flow-sink-node">
      <xsl:text> TO SINK </xsl:text>
      <xsl:value-of select="$wf-flow-sink-node/@name"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:text> TO </xsl:text>
      <xsl:call-template name="singlequote">
  <xsl:with-param name="string" select="$my-target"/>
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>
  <xsl:value-of select='$eol'/>
  <xsl:apply-templates
                   select="$wf-data-link/wf:presentation/wfpres:symbol">
    <xsl:with-param name="ini" select="$ini"/>
  </xsl:apply-templates>
</xsl:template>

<xsl:template match="wsfl:map" mode="emit-data-link-map">
  <xsl:param name="ini"/>
  <xsl:if test="@sourcePart and @targetPart">
    <xsl:value-of select="concat($ini, $indent)"/>
    <xsl:text>MAP </xsl:text>
    <xsl:call-template name="singlequote">
      <xsl:with-param name="string" select="@sourcePart"/>
    </xsl:call-template>
    <xsl:text> TO </xsl:text>
    <xsl:call-template name="singlequote">
      <xsl:with-param name="string" select="@targetPart"/>
    </xsl:call-template>
  </xsl:if>
  <xsl:value-of select='$eol'/>
</xsl:template>

<xsl:template name="emit-default-data-link-map">
  <xsl:param name="ini"/>
  <xsl:value-of select="concat($ini, $indent)"/>
  <xsl:text>MAP '_STRUCT' TO '_STRUCT'</xsl:text>
  <xsl:value-of select='$eol'/>
</xsl:template>

<!-- ********************************************************* -->
<!-- * CONTROLLINK                                           * -->
<!-- ********************************************************* -->

<xsl:template match="wsfl:controlLink">
  <xsl:param name="ini"/>
  <xsl:value-of select="$ini"/>
  <xsl:text>CONTROL </xsl:text>
  <xsl:value-of select='$eol'/>
  <xsl:variable name="my-name" select="./@name"/>
  <xsl:call-template name="control-link-settings">
    <xsl:with-param name="ini" select="$ini"/>
    <xsl:with-param name="wf-control-link"
        select="//wf:deployment//controlLink[@name=$my-name]/wf:controlLink"/>
  </xsl:call-template>
</xsl:template>

<xsl:template name="control-link-settings">
  <xsl:param name="ini"/>
  <xsl:param name="wf-control-link"/>
  <xsl:if test="@description">
    <xsl:value-of select="concat($ini, $indent)"/>
    <xsl:text>DESCRIPTION </xsl:text>
    <xsl:value-of select="@description"/>
    <xsl:value-of select='$eol'/>
  </xsl:if>
  <xsl:if test="@source">
    <xsl:value-of select="concat($ini, $indent)"/>
    <xsl:text>FROM </xsl:text>
    <xsl:call-template name="singlequote">
      <xsl:with-param name="string">
  <xsl:call-template name="removePrefix">
    <xsl:with-param name="qname" select="@source"/>
  </xsl:call-template>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:if>
  <xsl:if test="@target">
    <xsl:text> TO </xsl:text>
    <xsl:call-template name="singlequote">
      <xsl:with-param name="string">
  <xsl:call-template name="removePrefix">
    <xsl:with-param name="qname" select="@target"/>
  </xsl:call-template>
      </xsl:with-param>
    </xsl:call-template>
    <xsl:value-of select='$eol'/>
  </xsl:if>
  <xsl:if test="not(@transitionCondition)">
    <xsl:value-of select="concat($ini, $indent)"/>
    <xsl:text>OTHERWISE</xsl:text>
    <xsl:value-of select='$eol'/>
  </xsl:if>
  <xsl:if test="@transitionCondition">
    <xsl:value-of select="concat($ini, $indent)"/>
    <xsl:text>WHEN </xsl:text>
    <xsl:call-template name="doublequote">
      <xsl:with-param name="string" select="@transitionCondition"/>
    </xsl:call-template>
    <xsl:value-of select='$eol'/>
  </xsl:if>
  <xsl:apply-templates
                   select="$wf-control-link/wf:presentation/wfpres:symbol">
    <xsl:with-param name="ini" select="$ini"/>
  </xsl:apply-templates>
</xsl:template>

</xsl:stylesheet>
