<?xml version='1.0' ?>

<xsl:stylesheet xmlns:xsl  = 'http://www.w3.org/1999/XSL/Transform'
                xmlns:wsfl = 'http://schemas.xmlsoap.org/wsfl'
                xmlns:wsdl = 'http://schemas.xmlsoap.org/wsdl'
                xmlns:wf   = 'http://www.ibm.com/workflow/2001'
                xmlns:wfpi = 'http://www.ibm.com/workflow/2001/programInvocation'
                xmlns:exe  = 'http://schemas.xmlsoap.org/wsdl/exe'
                version    = '1.0'>

<xsl:import href='xsl:/common.xsl'/>

<!-- ********************************************************* -->
<!-- * PROGRAM                                               * -->
<!-- ********************************************************* -->

<xsl:template match="wsfl:internal" mode="emit-program">
 <xsl:param name="procOption"/>
 <xsl:if test="$procOption='Create'">
  <xsl:text>CREATE&#x20;</xsl:text>
 </xsl:if>
 <xsl:if test="$procOption='Update'">
  <xsl:text>UPDATE&#x20;</xsl:text>
 </xsl:if>
 <xsl:if test="$procOption='Replace'">
  <xsl:text>REPLACE&#x20;</xsl:text>
 </xsl:if>
 <xsl:variable name="my-sp">
  <xsl:call-template name="removePrefix">
   <xsl:with-param name="qname" select="@serviceProvider"/>
  </xsl:call-template>
 </xsl:variable>
 <xsl:variable name="my-pt">
  <xsl:call-template name="removePrefix">
   <xsl:with-param name="qname" select="@portType"/>
  </xsl:call-template>
 </xsl:variable>
 <xsl:variable name="my-opn">
  <xsl:call-template name="removePrefix">
   <xsl:with-param name="qname" select="@operation"/>
  </xsl:call-template>
 </xsl:variable>
 <xsl:variable name="service-provider" select="//wf:deployment//wsfl:serviceProvider[@name=$my-sp]"/>
 <xsl:variable name="source" select="$service-provider//wf:locator/@source"/>
 <xsl:variable name="int-prog-doc" select="document('xsl:./internal.xsl')"/>
 <xsl:variable name="exe-prog-url">
  <xsl:choose>
   <xsl:when test="string($source)='EXE'">
    <xsl:value-of select="$service-provider//wf:locator/@url"/>
   </xsl:when>
   <xsl:otherwise>
    <xsl:value-of select="'xsl:./internal.xsl'"/>
   </xsl:otherwise>
  </xsl:choose>
 </xsl:variable>
 <xsl:variable name="exe-prog-doc" select="document($exe-prog-url, /)"/>
 <xsl:variable name="service-name">
  <xsl:call-template name="removePrefix">
   <xsl:with-param name="qname" select="$service-provider//wf:locator/@service"/>
  </xsl:call-template>
 </xsl:variable>
 <xsl:variable name="service" select="$exe-prog-doc//wsdl:service[@name=$service-name]"/>
 <xsl:variable name="binding-name">
  <xsl:call-template name="removePrefix">
   <xsl:with-param name="qname" select="$service/wsdl:port[@name=$my-pt]/@binding"/>
  </xsl:call-template>
 </xsl:variable>
 <xsl:variable name="binding" select="$exe-prog-doc//wsdl:binding[@name=$binding-name]"/>
 <xsl:variable name="w-b-opn" select="$int-prog-doc//wsdl:binding/wsdl:operation"/>
 <xsl:variable name="e-b-opn" select="$binding/wsdl:operation[@name=$my-opn]"/>
 <xsl:text>PROGRAM </xsl:text>
 <xsl:call-template name="singlequote">
  <xsl:with-param name="string" select="$my-opn"/>
 </xsl:call-template>
 <xsl:call-template name="my-signature">
  <xsl:with-param name="input" select="../../wsdl:input/@message"/>
  <xsl:with-param name="output" select="../../wsdl:output/@message"/>
 </xsl:call-template>
 <xsl:value-of select='$eol'/>
 <xsl:choose>
  <xsl:when test="string($source)='WSDL'">
   <xsl:apply-templates select="$w-b-opn/wfpi:operation" mode="emit-program">
    <xsl:with-param name="ini" select="$indent"/>
   </xsl:apply-templates>
  </xsl:when>
  <xsl:otherwise>
   <xsl:apply-templates select="$e-b-opn/wfpi:operation" mode="emit-program">
    <xsl:with-param name="ini" select="$indent"/>
   </xsl:apply-templates>
  </xsl:otherwise>
 </xsl:choose>
 <xsl:variable name="opn-parm">
  <xsl:if test="string($source)='WSDL'">
   <xsl:call-template name="prog-parm">
    <xsl:with-param name="service-provider" select="$service-provider"/>
   </xsl:call-template>
  </xsl:if>
 </xsl:variable>
 <xsl:choose>
  <xsl:when test="string($source)='WSDL'">
   <xsl:apply-templates select="$w-b-opn/exe:operation">
    <xsl:with-param name="ini" select="$indent"/>
    <xsl:with-param name="opn-parm" select="$opn-parm"/>
   </xsl:apply-templates>
  </xsl:when>
  <xsl:otherwise>
   <xsl:apply-templates select="$e-b-opn/exe:operation">
    <xsl:with-param name="ini" select="$indent"/>
    <xsl:with-param name="opn-parm" select="$opn-parm"/>
   </xsl:apply-templates>
  </xsl:otherwise>
 </xsl:choose>
 <xsl:text>END </xsl:text>
 <xsl:call-template name="singlequote">
  <xsl:with-param name="string" select="$my-opn"/>
 </xsl:call-template>
 <xsl:value-of select='$eol'/>
 <xsl:value-of select='$eol'/>
</xsl:template>

<!-- ********************************************************* -->
<!-- * program settings                                      * -->
<!-- ********************************************************* -->

<xsl:template match="wfpi:operation" mode="emit-program">
  <xsl:param name="ini"/>
  <xsl:if test="@structuresFromActivity">
    <xsl:value-of select="$ini"/>
    <xsl:if test="@structuresFromActivity='no'">
      <xsl:text>NO </xsl:text>
    </xsl:if>
    <xsl:text>STRUCTURES_FROM_ACTIVITY</xsl:text>
    <xsl:value-of select='$eol'/>
  </xsl:if>
  <xsl:if test="@unattended">
    <xsl:value-of select="$ini"/>
    <xsl:if test="string(@unattended)='no'">
      <xsl:text>NOT </xsl:text>
    </xsl:if>
    <xsl:text>UNATTENDED</xsl:text>
    <xsl:value-of select='$eol'/>
  </xsl:if>
  <xsl:if test="@executionMode">
    <xsl:value-of select="$ini"/>
    <xsl:text>EXECUTION_MODE </xsl:text>
    <xsl:if test="string(@executionMode)='normal'">
      <xsl:text>NORMAL</xsl:text>
    </xsl:if>
    <xsl:if test="string(@executionMode)='safe'">
      <xsl:text>SAFE</xsl:text>
    </xsl:if>
    <xsl:value-of select='$eol'/>
  </xsl:if>
  <xsl:if test="@trusted">
    <xsl:value-of select="$ini"/>
    <xsl:if test="string(@trusted)='no'">
      <xsl:text>NOT </xsl:text>
    </xsl:if>
    <xsl:text>TRUSTED</xsl:text>
    <xsl:value-of select='$eol'/>
  </xsl:if>
  <xsl:if test="@executionUser">
    <xsl:value-of select="$ini"/>
    <xsl:text>EXECUTION_USER </xsl:text>
    <xsl:if test="string(@executionUser)='starter'">
      <xsl:text>STARTER</xsl:text>
    </xsl:if>
    <xsl:if test="string(@executionUser)='agent'">
      <xsl:text>AGENT</xsl:text>
    </xsl:if>
    <xsl:value-of select='$eol'/>
  </xsl:if>
  <xsl:if test="@inputContainerAccess">
    <xsl:value-of select="$ini"/>
    <xsl:if test="string(@inputContainerAccess)='no'">
      <xsl:text>NO </xsl:text>
    </xsl:if>
    <xsl:text>INPUT_CONTAINER_ACCESS</xsl:text>
    <xsl:value-of select='$eol'/>
  </xsl:if>
  <xsl:if test="@outputContainerAccess">
    <xsl:value-of select="$ini"/>
    <xsl:if test="string(@outputContainerAccess)='no'">
      <xsl:text>NO </xsl:text>
    </xsl:if>
    <xsl:text>OUTPUT_CONTAINER_ACCESS</xsl:text>
    <xsl:value-of select='$eol'/>
  </xsl:if>
</xsl:template>

<!-- ********************************************************* -->
<!-- * IMPLEMENTATION (program invocation)                   * -->
<!-- ********************************************************* -->

<xsl:template match="exe:operation">
  <xsl:param name="ini"/>
  <xsl:param name="opn-parm"/>
  <xsl:variable name="subitem" select="concat($ini, $indent)"/>
  <xsl:value-of select="$ini"/>
  <xsl:value-of select="@os"/>
  <xsl:text> </xsl:text>
  <xsl:value-of select="@type"/>
  <xsl:text> </xsl:text>
  <xsl:if test ="@pathAndFileName">
    <xsl:text>PATH_AND_FILENAME </xsl:text>
    <xsl:call-template name="doublequote">
      <xsl:with-param name="string" select="@pathAndFileName"/>
    </xsl:call-template>
  </xsl:if>
  <xsl:value-of select='$eol'/>
  <xsl:variable name="my-parm">
    <xsl:choose>
      <xsl:when test="string-length($opn-parm) > 0">
        <xsl:value-of select="$opn-parm"/>
      </xsl:when>
      <xsl:when test="@parameter">
        <xsl:value-of select="@parameter"/>
      </xsl:when>
    </xsl:choose>
  </xsl:variable>
  <xsl:if test ="string-length($my-parm) > 0">
    <xsl:value-of select="$subitem"/>
    <xsl:text>PARAMETER </xsl:text>
    <xsl:call-template name="doublequote">
      <xsl:with-param name="string" select="$my-parm"/>
    </xsl:call-template>
    <xsl:value-of select='$eol'/>
  </xsl:if>
  <xsl:if test ="@workingDirectoryName">
    <xsl:value-of select="$subitem"/>
    <xsl:text>WORKING_DIRECTORY </xsl:text>
    <xsl:call-template name="doublequote">
      <xsl:with-param name="string" select="@workingDirectoryName"/>
    </xsl:call-template>
    <xsl:value-of select='$eol'/>
  </xsl:if>
  <xsl:if test ="@inheritEnvironment">
    <xsl:value-of select="$subitem"/>
    <xsl:if test="string(@inheritEnvironment)='no'">
      <xsl:text>DO NOT </xsl:text>
    </xsl:if>
    <xsl:text>INHERIT ENVIRONMENT</xsl:text>
    <xsl:value-of select='$eol'/>
  </xsl:if>
  <xsl:if test ="@environment">
    <xsl:value-of select="$subitem"/>
    <xsl:text>ENVIRONMENT </xsl:text>
    <xsl:call-template name="singlequote">
      <xsl:with-param name="string" select="@environment"/>
    </xsl:call-template>
    <xsl:value-of select='$eol'/>
  </xsl:if>
  <!-- ! -->
  <xsl:if test ="@style">
    <xsl:value-of select="$subitem"/>
    <xsl:text>STYLE </xsl:text>
    <xsl:if test="string(@style)='visible'">
      <xsl:text>VISIBLE</xsl:text>
    </xsl:if>
    <xsl:if test="string(@style)='invisible'">
      <xsl:text>INVISIBLE</xsl:text>
    </xsl:if>
    <xsl:if test="string(@style)='maximized'">
      <xsl:text>MAXIMIZED</xsl:text>
    </xsl:if>
    <xsl:if test="string(@style)='minimized'">
      <xsl:text>MINIMIZED</xsl:text>
    </xsl:if>
    <xsl:value-of select='$eol'/>
  </xsl:if>

  <xsl:if test ="@startInForeGround">
    <xsl:value-of select="$subitem"/>
    <xsl:if test="string(@startInForeGround)='no'">
      <xsl:text>NO </xsl:text>
    </xsl:if>
    <xsl:text>START FOREGROUND</xsl:text>
    <xsl:value-of select='$eol'/>
  </xsl:if>
  <xsl:if test ="@runInXTerm">
    <xsl:value-of select="$subitem"/>
    <xsl:if test="string(@runInXTerm)='no'">
      <xsl:text>DO NOT </xsl:text>
    </xsl:if>
    <xsl:text>RUN_IN_XTERM</xsl:text>
    <xsl:value-of select='$eol'/>
  </xsl:if>
  <xsl:if test ="@automaticClose">
    <xsl:value-of select="$subitem"/>
    <xsl:if test="string(@automaticClose)='no'">
      <xsl:text>NO </xsl:text>
    </xsl:if>
    <xsl:text>AUTOCLOSE</xsl:text>
    <xsl:value-of select='$eol'/>
  </xsl:if>
  <xsl:if test ="@entryPointName">
    <xsl:value-of select="$subitem"/>
    <xsl:text>ENTRY_POINT </xsl:text>
    <xsl:call-template name="singlequote">
      <xsl:with-param name="string" select="@entryPointName"/>
    </xsl:call-template>
    <xsl:value-of select='$eol'/>
  </xsl:if>
  <xsl:if test ="@executeFenced">
    <xsl:value-of select="$subitem"/>
    <xsl:if test="string(@executeFenced)='no'">
      <xsl:text>NO </xsl:text>
    </xsl:if>
    <xsl:text>FENCED_MODE</xsl:text>
    <xsl:value-of select='$eol'/>
  </xsl:if>
  <xsl:if test ="@keepLoaded">
    <xsl:value-of select="$subitem"/>
    <xsl:if test="string(@keepLoaded)='no'">
      <xsl:text>DO NOT </xsl:text>
    </xsl:if>
    <xsl:text>KEEP_LOADED</xsl:text>
    <xsl:value-of select='$eol'/>
  </xsl:if>
  <xsl:if test ="@dllV2Compatibility">
    <xsl:value-of select="$subitem"/>
    <xsl:if test="string(@dllV2Compatibility)='no'">
      <xsl:text>NO </xsl:text>
    </xsl:if>
    <xsl:text>DLL_V2_COMPATIBILITY</xsl:text>
    <xsl:value-of select='$eol'/>
  </xsl:if>
</xsl:template>

<xsl:template name="prog-parm">
  <xsl:param name="service-provider"/>
  <xsl:variable name="wf-loc"
            select="$service-provider/wsfl:locator[@type='local']/wf:locator"/>
  <xsl:variable name="loc-svc">
    <xsl:call-template name="removePrefix">
      <xsl:with-param name="qname" select="$wf-loc/@service"/>
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="my-pt">
    <xsl:call-template name="removePrefix">
      <xsl:with-param name="qname" select="@portType"/>
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="my-opn">
    <xsl:call-template name="removePrefix">
      <xsl:with-param name="qname" select="@operation"/>
    </xsl:call-template>
  </xsl:variable>
  <xsl:value-of select="concat('SOURCE=',$wf-loc/@source,
             ';DOCUMENT=',$wf-loc/@url,
             ';SERVICE=',$loc-svc,
             ';PORT=',$my-pt,
             ';OPERATION=',$my-opn)"/>
</xsl:template>

</xsl:stylesheet>
