<?xml version='1.0' ?>

<!-- *********************************************************** -->
<!-- * DESCRIPTION:                                            * -->
<!-- *   Main stylesheet for the translation from WSFL to FDL  * -->
<!-- * AUTHOR : Ignacio Silva-Lepe                             * -->
<!-- *          with input from Andreas Fried's FDML to FDL    * -->
<!-- *          transformation stylesheets                     * -->
<!-- * VERSION: 0.1                                            * -->
<!-- * DATE   : 2001-06-21                                     * -->
<!-- * FILE   : wsfl2fdl.xls                                   * -->
<!-- *********************************************************** -->

<xsl:stylesheet xmlns:xsl      = 'http://www.w3.org/1999/XSL/Transform'
                xmlns:wsfl     = 'http://schemas.xmlsoap.org/wsfl'
                xmlns:wsdl     = 'http://schemas.xmlsoap.org/wsdl'
                xmlns:wf       = 'http://www.ibm.com/workflow/2001'
                xmlns:lxslt    = 'http://xml.apache.org/xslt'
                xmlns:java     = 'http://xml.apache.org/xslt/java'
                xmlns:redirect = 'xalan://org.apache.xalan.lib.Redirect'
                extension-element-prefixes='redirect'
                version = '1.0'>

<!-- ********************************************************* -->
<!-- * Import of nested stylesheets                          * -->
<!-- ********************************************************* -->

<xsl:import href='xsl:./person.xsl'/>
<xsl:import href='xsl:./structure.xsl'/>
<xsl:import href='xsl:./program.xsl'/>
<xsl:import href='xsl:./server.xsl'/>
<xsl:import href='xsl:./process.xsl'/>

<xsl:param name='generator'/>
<xsl:param name='input'/>
<xsl:param name='outdir'/>
<xsl:param name='options'/>

<xsl:output method='text'/>

<xsl:template match='/'>
 <xsl:apply-templates select='wsfl:definitions'/>
</xsl:template>

<!-- ********************************************************* -->
<!-- * DECLARATION                                           * -->
<!-- * Main entry for transforming FDL elements              * -->
<!-- ********************************************************* -->

<xsl:template match='wsfl:definitions'>
 <xsl:variable name='fileName' select='concat($outdir, @name, ".fdl")'/>
 <redirect:write select='$fileName'>
  <xsl:text>// MQSeries Workflow FDL for WSFL definitions '</xsl:text>
  <xsl:value-of select='@name'/>
  <xsl:text>'</xsl:text>
  <xsl:value-of select='$eol'/>
  <xsl:text>// This file ("</xsl:text>
  <xsl:variable name='javaFile' select='java:java.io.File.new($fileName)'/>
  <xsl:variable name='absoluteFileName' select='java:getAbsolutePath($javaFile)'/>
  <xsl:value-of select='$absoluteFileName'/>
  <xsl:text>")</xsl:text>
  <xsl:value-of select='$eol'/>
  <xsl:text>// was generated by </xsl:text>
  <xsl:value-of select='$generator'/>
  <xsl:text> on </xsl:text>
  <xsl:value-of select='java:java.util.Date.new()'/>
  <xsl:value-of select='$eol'/>
  <xsl:text>// from WSFL file "</xsl:text>
  <xsl:value-of select='$input'/>
  <xsl:text>"</xsl:text>
  <xsl:value-of select='$eol'/>
  <xsl:value-of select='$eol'/>
  <xsl:value-of select='$codepage'/>
  <xsl:value-of select='$eol'/>
  <xsl:value-of select='$fmrelease'/>
  <xsl:value-of select='$eol'/>
  <xsl:value-of select='$eol'/>
  <xsl:call-template name='heading'>
   <xsl:with-param name='str' select='"PERSONs"'/>
  </xsl:call-template>
  <xsl:apply-templates select='wf:deployment/wf:person'/>
  <xsl:call-template name='heading'>
   <xsl:with-param name='str' select='"STRUCTUREs"'/>
  </xsl:call-template>
  <xsl:apply-templates select='.' mode='emit-structures'/>
  <xsl:call-template name='heading'>
   <xsl:with-param name='str' select='"PROGRAMs"'/>
  </xsl:call-template>
  <xsl:apply-templates select='wsfl:flowModel/wsfl:activity//wsfl:internal' mode='emit-program'/>
  <xsl:call-template name='heading'>
   <xsl:with-param name='str' select='"SERVERs"'/>
  </xsl:call-template>
  <xsl:apply-templates select='wf:deployment/wf:server'/>
  <xsl:call-template name='heading'>
   <xsl:with-param name='str' select='"PROCESSes"'/>
  </xsl:call-template>
  <xsl:apply-templates select='wsfl:flowModel'/>
 </redirect:write>
</xsl:template>

<xsl:template match='wsfl:definitions' mode='emit-structures'>
 <xsl:variable name='insNouts' select='.//wsdl:input[@message] | .//wsdl:output[@message]'/>
 <xsl:variable name='msgs' select='$insNouts[not(@message=preceding::*/@message)]'/>
 <!-- <xsl:variable name='this' select='.'/> -->
 <xsl:variable name='imports' select='wsfl:import'/>
 <xsl:for-each select='$msgs/@message'>
  <xsl:variable name='lname'>
   <xsl:call-template name='removePrefix'>
    <xsl:with-param name='qname' select='.'/>
   </xsl:call-template>
  </xsl:variable>
  <!-- <xsl:apply-templates select='$this/wsfl:message[@name=$lname]'/> -->
  <xsl:call-template name='emit-struct-for-message'>
   <xsl:with-param name='msg-name' select='$lname'/>
   <xsl:with-param name='imports' select='$imports'/>
  </xsl:call-template>
 </xsl:for-each>
</xsl:template>

<xsl:template name='emit-struct-for-message'>
 <xsl:param name='msg-name'/>
 <xsl:param name='imports'/>
 <xsl:if test='$imports'>
  <xsl:variable name='import' select='document($imports[1]/@location, /)'/>
  <xsl:variable name='msg' select='$import//wsdl:message[@name=$msg-name]'/>
  <xsl:choose>
   <xsl:when test='$msg'>
    <xsl:apply-templates select='$msg'/>
   </xsl:when>
   <xsl:otherwise>
    <xsl:call-template name='emit-struct-for-message'>
     <xsl:with-param name='msg-name' select='$msg-name'/>
     <xsl:with-param name='imports' select='$imports[position() &gt; 1]'/>
    </xsl:call-template>
   </xsl:otherwise>
  </xsl:choose>
 </xsl:if>
</xsl:template>

</xsl:stylesheet>
