<?xml version='1.0'?>

<xsl:stylesheet xmlns:xsl   = 'http://www.w3.org/1999/XSL/Transform'
                xmlns:lxslt = 'http://xml.apache.org/xslt'
                xmlns:java  = 'http://xml.apache.org/xslt/java'
                xmlns:wf    = 'http://www.ibm.com/workflow/2001'
                version='1.0'>

<!-- ********************************************************* -->
<!-- * Global variables                                      * -->
<!-- ********************************************************* -->

<xsl:variable name='eol'       select='"&#xA;"'/>
<xsl:variable name="indent"    select='"&#x20;&#x20;"'/>
<xsl:variable name="indent1"   select='"&#x20;&#x20;"'/>
<xsl:variable name="indent2"   select='"&#x20;&#x20;&#x20;&#x20;"'/>
<xsl:variable name='codepage'  select='"CODEPAGE   1252"'/>
<xsl:variable name='fmrelease' select='"FM_RELEASE V3R3 2"'/>
<xsl:variable name='version'   select='"3"'/>
<xsl:variable name='release'   select='"3"'/>
<xsl:variable name='level'     select='"2"'/>

<!-- ********************************************************* -->
<!-- * Commonly used functions follow                        * -->
<!-- ********************************************************* -->

<!-- ********************************************************* -->
<!-- * anystring -> 'anystring'                              * -->
<!-- ********************************************************* -->

<xsl:template name="singlequote">
  <xsl:param name="string"/>
  <xsl:value-of select='concat("&apos;", $string, "&apos;")'/>
</xsl:template>


<!-- ********************************************************* -->
<!-- * anystring -> "anystring"                              * -->
<!-- ********************************************************* -->

<xsl:template name="doublequote">
  <xsl:param name="string"/>
  <xsl:value-of select="concat('&quot;', $string, '&quot;')"/>
</xsl:template>


<!-- ********************************************************* -->
<!-- * anystring -> (anystring)                              * -->
<!-- ********************************************************* -->

<xsl:template name="parenthesize">
  <xsl:param name="string"/>
  <xsl:value-of select="concat('(', $string, ')')"/>
</xsl:template>


<!-- ********************************************************* -->
<!-- * converts a string to the upper case                   * -->
<!-- ********************************************************* -->

<xsl:template name="toUpperCase">
  <xsl:param name="string"/>
  <xsl:value-of select="translate($string, 'abcdefghijklmnopqrstuvwxyz&#xe4;&#xf6;&#xfc;', 'ABCDEFGHIJKLMNOPQRSTUVWXYZ&#xc4;&#xd6;&#xdc;')"/>
</xsl:template>


<!-- ********************************************************* -->
<!-- * converts a string to the lower case                   * -->
<!-- ********************************************************* -->

<xsl:template name="toLowerCase">
  <xsl:param name="string"/>
  <xsl:value-of select="translate($string, 'ABCDEFGHIJKLMNOPQRSTUVWXYZ&#xc4;&#xd6;&#xdc;', 'abcdefghijklmnopqrstuvwxyz&#xe4;&#xf6;&#xfc;')"/>
</xsl:template>


<!-- ********************************************************* -->
<!-- * adds the name, signature                              * -->
<!-- ********************************************************* -->

<xsl:template name="ndds">
  <xsl:param name="ini"/>
  <xsl:if test="@name">
    <xsl:call-template name="singlequote">
      <xsl:with-param name="string" select="@name"/>
    </xsl:call-template>
  </xsl:if>
  <xsl:apply-templates select="child::wf:Signature"/>
  <xsl:value-of select='$eol'/>
</xsl:template>


<!-- ********************************************************* -->
<!-- * input/output                                          * -->
<!-- ********************************************************* -->

<xsl:template name="signature">
  <xsl:text> (</xsl:text>
  <xsl:if test="@input!=''">
    <xsl:call-template name="singlequote">
      <xsl:with-param name="string" select="@input"/>
    </xsl:call-template>
  </xsl:if>
  <xsl:text>, </xsl:text>
  <xsl:if test="@output!=''">
    <xsl:call-template name="singlequote">
      <xsl:with-param name="string" select="@output"/>
    </xsl:call-template>
  </xsl:if>
  <xsl:text>)</xsl:text>
</xsl:template>

<xsl:template name="my-signature">
  <xsl:param name="input"/>
  <xsl:param name="output"/>
  <xsl:variable name="my-input">
    <xsl:call-template name="removePrefix">
      <xsl:with-param name="qname" select="$input"/>
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="my-output">
    <xsl:call-template name="removePrefix">
      <xsl:with-param name="qname" select="$output"/>
    </xsl:call-template>
  </xsl:variable>
  <xsl:text> (</xsl:text>
  <xsl:if test="$input">
    <xsl:call-template name="singlequote">
      <xsl:with-param name="string" select="$my-input"/>
    </xsl:call-template>
  </xsl:if>
  <xsl:if test="not($input)">
    <xsl:call-template name="singlequote">
      <xsl:with-param name="string" select="'Default Data Structure'"/>
    </xsl:call-template>
  </xsl:if>
  <xsl:text>, </xsl:text>
  <xsl:if test="$output">
    <xsl:call-template name="singlequote">
      <xsl:with-param name="string" select="$my-output"/>
    </xsl:call-template>
  </xsl:if>
  <xsl:if test="not($output)">
    <xsl:call-template name="singlequote">
      <xsl:with-param name="string" select="'Default Data Structure'"/>
    </xsl:call-template>
  </xsl:if>
  <xsl:text>)</xsl:text>
</xsl:template>


<!-- ********************************************************* -->
<!-- *                                                       * -->
<!-- ********************************************************* -->

<xsl:template name="timeInterval">
  <xsl:if test="@years">
    <xsl:value-of select="@years"/>
    <xsl:text> YEARS </xsl:text>
  </xsl:if>
  <xsl:if test="@months">
    <xsl:value-of select="@months"/>
    <xsl:text> MONTHS </xsl:text>
  </xsl:if>
  <xsl:if test="@weeks">
    <xsl:value-of select="@weeks"/>
    <xsl:text> WEEKS </xsl:text>
  </xsl:if>
  <xsl:if test="@days">
    <xsl:value-of select="@days"/>
    <xsl:text> DAYS </xsl:text>
  </xsl:if>
  <xsl:if test="@hours">
    <xsl:value-of select="@hours"/>
    <xsl:text> HOURS </xsl:text>
  </xsl:if>
  <xsl:if test="@minutes">
    <xsl:value-of select="@minutes"/>
    <xsl:text> MINUTES </xsl:text>
  </xsl:if>
  <xsl:if test="@seconds">
    <xsl:value-of select="@seconds"/>
    <xsl:text> SECONDS</xsl:text>
  </xsl:if>
</xsl:template>


<!-- ********************************************************* -->
<!-- * Commonly used templates                               * -->
<!-- ********************************************************* -->

<!-- ********************************************************* -->
<!-- *                                                       * -->
<!-- ********************************************************* -->

<xsl:template name="inputContainer" match="wf:InputContainer">
  <xsl:param name="ini"/>
  <xsl:value-of select="$ini"/>
  <xsl:text>INPUT_CONTAINER</xsl:text>
  <xsl:value-of select='$eol'/>
  <xsl:apply-templates select="child::wf:ContainerEntry">
    <xsl:with-param name="ini" select="$ini"/>
  </xsl:apply-templates>
</xsl:template>


<!-- ********************************************************* -->
<!-- *                                                       * -->
<!-- ********************************************************* -->

<xsl:template name="outputContainer" match="wf:OutputContainer">
  <xsl:param name="ini"/>
  <xsl:value-of select="$ini"/>
  <xsl:text>OUTPUT_CONTAINER</xsl:text>
  <xsl:value-of select='$eol'/>
  <xsl:apply-templates select="child::wf:ContainerEntry">
    <xsl:with-param name="ini" select="$ini"/>
  </xsl:apply-templates>
</xsl:template>


<!-- ********************************************************* -->
<!-- *                                                       * -->
<!-- ********************************************************* -->

<xsl:template name="containerEntry" match="wf:ContainerEntry">
  <xsl:param name="ini"/>
  <xsl:value-of select="$ini"/>
  <xsl:value-of select="$indent"/>
  <xsl:call-template name="singlequote">
    <xsl:with-param name="string" select="@name"/>
  </xsl:call-template>
  <xsl:text>INITIAL_VALUE </xsl:text>
  <xsl:call-template name="singlequote">
    <xsl:with-param name="string" select="@initialValue"/>
  </xsl:call-template>
  <xsl:value-of select='$eol'/>
</xsl:template>


<!-- ********************************************************* -->
<!-- * DESCRIPTION                                           * -->
<!-- ********************************************************* -->

<xsl:template name="description" match="wf:description">
  <xsl:param name="ini"/>
  <xsl:value-of select="$ini"/>
  <xsl:text>DESCRIPTION </xsl:text>
  <xsl:call-template name="doublequote">
    <xsl:with-param name="string" select="."/>
  </xsl:call-template>
</xsl:template>


<!-- ********************************************************* -->
<!-- * DOCUMENTATION                                         * -->
<!-- ********************************************************* -->

<xsl:template name="documentation" match="wf:Documentation">
  <xsl:param name="ini"/>
  <xsl:value-of select="$ini"/>
  <xsl:text>DOCUMENTATION </xsl:text>
  <xsl:call-template name="singlequote">
    <xsl:with-param name="string" select="."/>
  </xsl:call-template>
</xsl:template>

<!-- ********************************************************* -->
<!-- * HEADING                                               * -->
<!-- ********************************************************* -->

<xsl:template name="heading">
  <xsl:param name="str"/>
  <xsl:text>/*</xsl:text>
  <xsl:value-of select='$eol'/>
  <xsl:text> *  </xsl:text>
  <xsl:value-of select="$str"/>
  <xsl:value-of select='$eol'/>
  <xsl:text> */</xsl:text>
  <xsl:value-of select='$eol'/>
</xsl:template>

<!-- ********************************************************* -->
<!-- * COORDS                                                * -->
<!-- ********************************************************* -->

<xsl:template name="coords">
  <xsl:param name="node"/>
  <xsl:text> XPOS </xsl:text>
  <xsl:value-of select="$node/@xpos"/>
  <xsl:text> YPOS </xsl:text>
  <xsl:value-of select="$node/@ypos"/>
</xsl:template>

</xsl:stylesheet>
