// *****************************************************************************
// * IBM MQSeries Workflow XML Sample Scenario                                 *
// *****************************************************************************
//
//  Licensed Materials - Property of IBM
//  5697-FM3 5655-A96
//  (C) Copyright IBM Corp. 1999-2000
//  US Government Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract
//  with IBM Corp.
//
// *****************************************************************************
// *****************************************************************************
//  DISCLAIMER
//  ----------
//  This material contains programming source code for your consideration.
//  These examples have not been thoroughly tested under all conditions.
//  IBM, therefore, cannot guarantee or imply reliability, serviceability,
//  or function of these programs.
//  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
//  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
//  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
//  FOR A PARTICULAR PURPOSE.
// *****************************************************************************
package com.ibm.workflow.upes;

import org.w3c.dom.Element;

/**
 * The <code>ImplementationData</code> class represents
 * the implementation data sections from the XML InvokeProgram
 * message of the MQSeries Workflow XML interface,
 * as a Java class.
 * 
 * For each possible entry in the XML message there is a data memeber of this class.
 * <p>
 * It has the ability to traverse a given DOM Tree (i.e. a given rootElement of type TXElement)
 * and to fill out all given information during the traversal.
 * <p>
 * @version 1.0
 */
public class ImplementationData
{
  public final int ExeOptions      = 1;
  public final int DllOptions      = 2;
  public final int ExternalOptions = 3;

  //KindOfOptions: {ExeOptions,DllOptions,ExternalOptions}
  public int KindOfOptions;

  public String _AutomaticClose;
  public String _BackwardMappingFormat;
  public String _BackwardMappingParameters;
  public String _CodePage;                 
  public String _DllV2Compatible;
  public String _EntryPointName;           
  public String _Environment;          
  public String _ExecutableName;         
  public String _ExecutableType;        
  public String _ExecuteFenced;       
  public String _ForwardMappingFormat;     
  public String _ForwardMappingParameters ;
  public String _InheritEnvironment;
  public String _InvocationType;
  public String _IsLocalUser;
  public String _IsMappingRoutineCall;
  public String _IsSecurityRoutineCall;
  public String _KeepLoaded;
  public String _MappingType;
  public String _PathAndFileName;
  public String _RunInXTerm;
  public String _ServiceName;
  public String _ServiceType;
  public String _StartInForeGround;
  public String _TimeoutInterval;
  public String _TimeoutPeriod;
  public String _WindowStyle;
  public String _WorkingDirectoryName;

/************************************************************
 * <b>ImplementationData</b> the constructor of the class, which parses the given root element 
 * of the DOM tree which represents the implementation data.
 *
 * @parameters <b>IDRoot</b> the root element of the DOM tree which represents the 
 * implementation data.
 * 
 ***********************************************************/

  public ImplementationData(Element IDRoot)
  {
    Element child;
    Element grandChild;
    //there are 3 different nested substructures within the XML message
    //ExeOptions
    //DllOptions 
    //ExternalOptions
    child = DOMUtils.getElementByTagName( IDRoot,"ExeOptions");
    if (child != null)
    {
      KindOfOptions = ExeOptions;
    }
    else
    {
      child = DOMUtils.getElementByTagName( IDRoot,"DllOptions");
      if (child != null)
      {
        KindOfOptions = DllOptions;
      }
      else
      {
        child = DOMUtils.getElementByTagName( IDRoot,"ExternalOptions");
        if (child != null)
        {
          KindOfOptions = ExternalOptions;
        }
      }
    }
    //****************************************************
    //for each attribute it is checked whether it is checked in the XML message
    grandChild = DOMUtils.getElementByTagName( child,"AutomaticClose");
    if (grandChild != null)
    {
      _AutomaticClose = DOMUtils.getChildCharacterData( grandChild);
    }
    grandChild = DOMUtils.getElementByTagName( child,"BackwardMappingFormat");
    if (grandChild != null)
    {
      _BackwardMappingFormat = DOMUtils.getChildCharacterData( grandChild);
    }
    grandChild = DOMUtils.getElementByTagName( child,"BackwardMappingParameters");
    if (grandChild != null)
    {
      _BackwardMappingParameters = DOMUtils.getChildCharacterData( grandChild);
    }
    grandChild = DOMUtils.getElementByTagName( child,"CodePage");
    if (grandChild != null)
    {
      _CodePage = DOMUtils.getChildCharacterData( grandChild);
    }
    grandChild = DOMUtils.getElementByTagName( child,"DllV2Compatible");
    if (grandChild != null)
    {
      _DllV2Compatible = DOMUtils.getChildCharacterData( grandChild);
    }
    grandChild = DOMUtils.getElementByTagName( child,"EntryPointName");
    if (grandChild != null)
    {
      _EntryPointName = DOMUtils.getChildCharacterData( grandChild);
    }
    grandChild = DOMUtils.getElementByTagName( child,"Environment");
    if (grandChild != null)
    {
      _Environment = DOMUtils.getChildCharacterData( grandChild);
    }
    grandChild = DOMUtils.getElementByTagName( child,"ExecutableName");
    if (grandChild != null)
    {
      _ExecutableName = DOMUtils.getChildCharacterData( grandChild);
    }
    grandChild = DOMUtils.getElementByTagName( child,"ExecutableType");
    if (grandChild != null)
    {
      _ExecutableType = DOMUtils.getChildCharacterData( grandChild);
    }
    grandChild = DOMUtils.getElementByTagName( child,"ExecuteFenced");
    if (grandChild != null)
    {
      _ExecuteFenced = DOMUtils.getChildCharacterData( grandChild);
    }
    grandChild = DOMUtils.getElementByTagName( child,"ForwardMappingFormat");
    if (grandChild != null)
    {
      _ForwardMappingFormat = DOMUtils.getChildCharacterData( grandChild);
    }
    grandChild = DOMUtils.getElementByTagName( child,"ForwardMappingParameters");
    if (grandChild != null)
    {
      _ForwardMappingParameters = DOMUtils.getChildCharacterData( grandChild);
    }
    grandChild = DOMUtils.getElementByTagName( child,"InheritEnvironment");
    if (grandChild != null)
    {
      _InheritEnvironment = DOMUtils.getChildCharacterData( grandChild);
    }
    grandChild = DOMUtils.getElementByTagName( child,"InvocationType");
    if (grandChild != null)
    {
      _InvocationType = DOMUtils.getChildCharacterData( grandChild);
    }
    grandChild = DOMUtils.getElementByTagName( child,"IsLocalUser");
    if (grandChild != null)
    {
      _IsLocalUser = DOMUtils.getChildCharacterData( grandChild);
    }
    grandChild = DOMUtils.getElementByTagName( child,"IsMappingRoutineCall");
    if (grandChild != null)
    {
      _IsMappingRoutineCall = DOMUtils.getChildCharacterData( grandChild);
    }
    grandChild = DOMUtils.getElementByTagName( child,"IsSecurityRoutineCall");
    if (grandChild != null)
    {
      _IsSecurityRoutineCall = DOMUtils.getChildCharacterData( grandChild);
    }
    grandChild = DOMUtils.getElementByTagName( child,"KeepLoaded");
    if (grandChild != null)
    {
      _KeepLoaded = DOMUtils.getChildCharacterData( grandChild);
    }
    grandChild = DOMUtils.getElementByTagName( child,"MappingType");
    if (grandChild != null)
    {
      _MappingType = DOMUtils.getChildCharacterData( grandChild);
    }
    grandChild = DOMUtils.getElementByTagName( child,"PathAndFileName");
    if (grandChild != null)
    {
      _PathAndFileName = DOMUtils.getChildCharacterData( grandChild);
    }
    grandChild = DOMUtils.getElementByTagName( child,"RunInXTerm");
    if (grandChild != null)
    {
      _RunInXTerm = DOMUtils.getChildCharacterData( grandChild);
    }
    grandChild = DOMUtils.getElementByTagName( child,"ServiceName");
    if (grandChild != null)
    {
      _ServiceName = DOMUtils.getChildCharacterData( grandChild);
    }
    grandChild = DOMUtils.getElementByTagName( child,"ServiceType");
    if (grandChild != null)
    {
      _ServiceType = DOMUtils.getChildCharacterData( grandChild);
    }
    grandChild = DOMUtils.getElementByTagName( child,"StartInForeGround");
    if (grandChild != null)
    {
      _StartInForeGround = DOMUtils.getChildCharacterData( grandChild);
    }
    grandChild = DOMUtils.getElementByTagName( child,"TimeoutInterval");
    if (grandChild != null)
    {
      _TimeoutInterval = DOMUtils.getChildCharacterData( grandChild);
    }
    grandChild = DOMUtils.getElementByTagName( child,"TimeoutPeriod");
    if (grandChild != null)
    {
      _TimeoutPeriod = DOMUtils.getChildCharacterData( grandChild);
    }
    grandChild = DOMUtils.getElementByTagName( child,"WindowStyle");
    if (grandChild != null)
    {
      _WindowStyle = DOMUtils.getChildCharacterData( grandChild);
    }
    grandChild = DOMUtils.getElementByTagName( child,"WorkingDirectoryName");
    if (grandChild != null)
    {
      _WorkingDirectoryName = DOMUtils.getChildCharacterData( grandChild);
    }
  }  
}      
