// *****************************************************************************
// * IBM MQSeries Workflow XML Sample Scenario                                 *
// *****************************************************************************
//
//  Licensed Materials - Property of IBM
//  5697-FM3 5655-A96
//  (C) Copyright IBM Corp. 1999-2000
//  US Government Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract
//  with IBM Corp.
//
// *****************************************************************************
// *****************************************************************************
//  DISCLAIMER
//  ----------
//  This material contains programming source code for your consideration.
//  These examples have not been thoroughly tested under all conditions.
//  IBM, therefore, cannot guarantee or imply reliability, serviceability,
//  or function of these programs.
//  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
//  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
//  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
//  FOR A PARTICULAR PURPOSE.
// *****************************************************************************
package com.ibm.workflow.upes;

import com.ibm.mq.*;            // Include the MQSeries classes for Java package
import org.w3c.dom.Element;
import java.lang.Byte;
import java.lang.Integer;
import java.lang.Runnable;
import java.lang.Thread;
import java.util.Enumeration;
import java.util.Hashtable;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import sun.io.ByteToCharUnicode;
import java.io.StringReader;
import org.apache.xerces.parsers.DOMParser;

/**
 * The <code>UPES</code> class represents an user defined
 * program execution server. This means that it reads messages from
 * a specified input queue and supports (registered) UPESListeners
 * with the recieved message.
 * <p>
 * An UPES listens to one input queue which must be specified with
 * queue name and queue manager name.
 * It is able to send messages to an arbitrary queue also specified
 * by queue and queue manager name.
 * <p>
 * UPESListener can register or deregister to the UPES so that when
 * a message arrives the UPES asks each listener if it is responsible
 * to handle the message. It then gives the message to the first listener
 * which claims the responsibility.
 * <p>
 * This is just a sample implementation of java UPES. This listener concept
 * is just one possible implementation.
 * @version 1.0
 */
public class UPES implements Runnable
{
    /************************************************************
     * <b>_Listeners</b> contain all registered UPESListeners
     *
     * @see java.util.Hashtable
     * @see UPESListener
     ***********************************************************/
    private Hashtable      _Listeners;
    /************************************************************
     * <b>_InputQName</b> name of the queue to which the UPES listens.
     *
     ***********************************************************/
    private String         _InputQName;
    /************************************************************
     * <b>_InputQMgrName</b> name of the queue manager to which the UPES listens.
     *
     ***********************************************************/
    private String          _InputQMgrName;
    public java.lang.String _log = "";
    protected transient java.util.Vector aChangeListener = null;
    private static int      _threadCount = 0;
    public  static synchronized int getThreadID() 
    {
        return ++_threadCount;
    }
    /************************************************************
    * <b>UPES</b> constructor which sets the queue name and queue manager name to which 
    * the UPES has to listen for incoming messages.
    * @parameters <b>QName</b> the name of the queue to which to listen for incoming messages
    * @parameters <b>QMgrName</b> the name of the queue manager to which to listen for incoming messages
    *
    ***********************************************************/
    public UPES(String QName, String QMgrName) throws MQException
    {
        _InputQMgrName = QMgrName;
        _InputQName    = QName;
        _Listeners     = new Hashtable();
        //Tracer.trace(" UPES() 1");
        //ConnectToInputQueue();
        //Tracer.trace(" UPES() 2");
    }   
    /**
     * Add a javax.swing.event.ChangeListener.
     */
    public void addChangeListener(javax.swing.event.ChangeListener newListener)
    {
        if(aChangeListener == null)
        {
            aChangeListener = new java.util.Vector();
        };
        aChangeListener.addElement(newListener);
    }
    /************************************************************
     * With <b>DeRegisterUPESListener</b> a already registered UPESListener can be deregistered.
     * @parameter <b>u</b> the upes listener which is to be deregistered
     * 
     ***********************************************************/
    public void DeRegisterUPESListener(UPESListener u)
    {
        if(_Listeners.contains(u))
        {
            Tracer.trace("Listener '"+u.getName()+"' DeRegistered ");
            _Listeners.remove(u);
        }
        Tracer.trace("Number of Listeners = "+_Listeners.size());
    }  
    /**
     * Method to support listener events.
     */
    protected void fireStateChanged(javax.swing.event.ChangeEvent e)
    {
        if(aChangeListener == null)
        {
            return;
        };
        int currentSize = aChangeListener.size();
        javax.swing.event.ChangeListener tempListener = null;
        for(int index = 0; index < currentSize; index++)
        {
            tempListener = (javax.swing.event.ChangeListener)aChangeListener.elementAt(index);
            if(tempListener != null)
            {
                tempListener.stateChanged(e);
            };
        };
    }
    /**
     * Insert the method's description here.
     * Creation date: (14.07.00 19:35:43)
     * @return java.lang.String
     */
    public java.lang.String get_log()
    {
        return _log;
    }
    public  String   getInputQMgrName()
    {
        return _InputQMgrName;
    }  
    public  String   getInputQName()
    {
        return _InputQName;
    }  
    /************************************************************
     * 
     * @parameters
     * @return  
     * 
     ***********************************************************/
    public static void helpMessage()
    {
        System.out.println("************************************");
        System.out.println("usage: java UPES <QName> [<QMName>] ");
        System.out.println("  <QName> - Queue Name of UPES Queue");
        System.out.println("  <QMName>- Queue Manager of UPES   ");
        System.out.println("            Queue (when omitted then");
        System.out.println("             Fastnet will be assumed");
        System.out.println("************************************"); 
    }  
    /************************************************************
     * 
     * @parameters
     * @return  
     * 
     ***********************************************************/
    public void ProcessIncommingXMLMessage(MQHandler MQH,String msg,String responseQ, String responseQMgr)
    {
        try
        {
            DOMParser parser = new DOMParser( );
            Tracer.trace("XML Parser Started");
            //Tracer.trace(msg);
            set_log(get_log()+"\n following message received \n"+msg); 
            StringReader  sb = new StringReader(msg);
            parser.parse(new InputSource(sb));
            Tracer.trace("XML Parser Ended");    
            Document document = parser.getDocument ();
            Element rootElement = (Element)document.getDocumentElement();
            InvokeProgramData invokeMsg = new InvokeProgramData(rootElement,responseQ,responseQMgr);
            UPESListener l = null;
            //Tracer.trace(invokeMsg._ProgramInputData.AsXMLString());
            //Tracer.trace(invokeMsg._ProgramOutputDataDefaults.AsXMLString());
            Enumeration enum = _Listeners.elements();
            //the first listener which returns a result object != null makes the deal
            while(l == null && enum.hasMoreElements())
            {
                UPESListener next = (UPESListener)enum.nextElement();
                if(next.IsActive() && next.IsResponsibleFor(invokeMsg))
                {
                    l = next;
                }
            }
            if(l == null)
            {
                Tracer.trace("error: No active Listener was responsible for the message");
                Tracer.trace("       Message remains in the Q ;-)");
                MQH.rollback();
            }
            else
            {
                // get will be committed	
                l.ProcessMessage(this,MQH,invokeMsg);
                MQH.commit();
            }
        }
        catch(Exception e)
        {
            Tracer.trace("error: "+e.getMessage());
            e.printStackTrace();
        }
    }  
    /************************************************************
     * With <b>RegisterUPESListener</b> an UPESListener can be registered.
     * @parameter <b>u</b> the upes listener which is to be registered
     * 
     ***********************************************************/
    public void RegisterUPESListener(UPESListener u)
    {
        Tracer.trace("Listener '"+u.getName()+"' Registered ");
        System.out.flush();
        _Listeners.put(u,u);
        Tracer.trace("Number of Listeners = "+_Listeners.size());
    }  
    /**
     * Remove a javax.swing.event.ChangeListener.
     */
    public void removeChangeListener(javax.swing.event.ChangeListener newListener)
    {
        if(aChangeListener != null)
        {
            aChangeListener.removeElement(newListener);
        };
    }
    /************************************************************
     * <b>run</b> is the actual polling and processing loop for the UPES.
     * It first gets an XML message from the input queue and then calls <b>'ProcessIncommingXMLMessage'</b>
     * to process the message. 
     * 
     ***********************************************************/
    public void run()
    {
        try
        {
            int threadCount = UPES.getThreadID();
            Tracer.trace("Thread "+threadCount+" started.");
            MQHandler MQH = new MQHandler(_InputQMgrName);
            while(true)
            {
                try
                {
                    MQMessage retrievedMessage = MQH.getMessage(_InputQMgrName,_InputQName);
                    Tracer.trace(" Message Length: " + Integer.toString(retrievedMessage.getDataLength(),10));
                    System.out.flush();
                    //transform the message buffer into a java String
                    byte b[] = new byte[retrievedMessage.getDataLength( ) ];
                    retrievedMessage.readFully( b, 0, retrievedMessage.getDataLength( ) );
                    String msgText = new String( b);
                    //process the message
                    Tracer.trace("---------------------------------------");
                    Tracer.trace(" Message RawString: " + msgText);
                    Tracer.trace("---------------------------------------");
                    System.out.flush();
                    ProcessIncommingXMLMessage(MQH,msgText,retrievedMessage.replyToQueueName,retrievedMessage.replyToQueueManagerName );
                }
                catch(Exception e)
                {
                    Tracer.trace("error: "+e.getMessage());
                    e.printStackTrace();
                }
            }
        }
        catch(Exception e)
        {
            Tracer.trace("error: "+e.getMessage());
            Tracer.trace("processing aborted in class: UPES method: run()");
            e.printStackTrace();
        }
    }  
    /************************************************************
     * With <b>SendMessage</b> the caller can send a invoke program response message to 
     * an arbitrary queue.
     * @parameters <b>response</b> the message which is to be sent
     * @parameters <b>QueueManagerName</b> the name of the queue manager to which the message is to be sent.
     * @parameters <b>QName</b> the name of the queue to which the message is to be sent.
     * 
     ***********************************************************/
    public void SendMessage(MQHandler MQH, InvokeProgramResponseData response, String QueueManagerName, String QName)
    {
        Tracer.trace("UPES.SendMessage entry");
        Tracer.trace("QueueManagerName="+QueueManagerName+",QName="+QName);

        String XMLResponse = response.GenerateXMLResponse();

        Tracer.trace("The response is: " + XMLResponse);
        set_log(get_log()+"\n sending following message to "+QueueManagerName+"/"+QName+"\n"+XMLResponse);  
        try
        {
            // prepare the data to put
            MQMessage msg2put = new MQMessage();

            msg2put.replyToQueueName = "ERROR";
            msg2put.replyToQueueManagerName = _InputQMgrName;
            msg2put.writeChars(XMLResponse);
            MQH.putMessage(msg2put,QueueManagerName,QName);
        }
        // If an error has occurred in the above, try to identify what went wrong
        // Was it an MQSeries error?
        catch(MQException ex)
        {
            Tracer.trace("An MQSeries error occurred : Completion code " +
                               ex.completionCode + " Reason code " + ex.reasonCode);
        }
        // Was it a Java buffer space error?
        catch(java.io.IOException ex)
        {
            Tracer.trace("An error occurred whilst writing to the message buffer: " + ex);
        }
        Tracer.trace("UPES.SendMessage exit");
    }  
    /**
     * Insert the method's description here.
     * Creation date: (14.07.00 19:35:43)
     * @param new_log java.lang.String
     */
    public void set_log(java.lang.String new_log)
    {
        _log = new_log;
        javax.swing.event.ChangeEvent c = new javax.swing.event.ChangeEvent(this);
        fireStateChanged(c);
    }
    public  void     setInputQMgrName(String s)
    {
        _InputQMgrName = s;
    }  
    public  void     setInputQName(String s)
    {
        _InputQName = s;
    }  
    /************************************************************
     * <b>WaitForMessages</b> opens a thread, which processes incoming messages 
     * 
     ***********************************************************/
    public void WaitForMessages(int numberOfThreads)
    {
        Tracer.trace("processing UPES.WaitForMessages");
        for(int i=0; i < numberOfThreads ; i++)
        {
            Thread t = new Thread(this);
            t.start();
        } //end for
    }  
}
