// *****************************************************************************
// * IBM MQSeries Workflow XML Sample Scenario                                 *
// *****************************************************************************
//
//  Licensed Materials - Property of IBM
//  5697-FM3 5655-A96
//  (C) Copyright IBM Corp. 1999-2000
//  US Government Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract
//  with IBM Corp.
//
// *****************************************************************************
// *****************************************************************************
//  DISCLAIMER
//  ----------
//  This material contains programming source code for your consideration.
//  These examples have not been thoroughly tested under all conditions.
//  IBM, therefore, cannot guarantee or imply reliability, serviceability,
//  or function of these programs.
//  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
//  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
//  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
//  FOR A PARTICULAR PURPOSE.
// *****************************************************************************
package com.ibm.workflow.upes;

/**
 * The <code>UPESListener</code> is an interface which should be implemented by
 * applications containing the actual UPES functionality.
 * Classes implementing this interface can register with an UPES.
 * The UPES uses the given interfaces for identifying for an incoming XML message
 * which listener is responsible for this message. The first listener which is
 * active (detected by the <b>IsActive()</b> method)
 * and claims the responsibility for the message
 * (detected by the <b>IsResponsibleFor(InvokeProgramData msg)</b> method) will be triggered
 * with a <b>ProcessMessage(UPES u,InvokeProgramData msg)</b> method call.
 * This sample implements only an interface to handle
 * InvokeProgram requests (this is interface provided by
 * MQSeries Workflow version 3.2.1 and higher)
 * @version 1.0
 * 
 * @see UPES
 */
public interface UPESListener
{
    /**
     * Just a name to distinguish different listeners
     * 
     * @return the name of the UPESListener
     */
    public String getName();
    /**
     * The method <b>IsActive</b> signals the caller whether the listener
     * is ready to accept messages or not. 
     * 
     * @return <b>true</b>, if the UPESListener is idle and ready to
     *         process messages by the ProcessMessage method.
     *         <br><b>false</b>,if the UPESListener is either busy or not
     *         ready to process messages.
     */
    public boolean IsActive();
    /**
     * The method <b>IsResponsible</b> signals the caller whether the listener
     * is responsible for a given message or not. Responsibility means
     * that the listener is able to process the message.
     * 
     * @param msg    contains all information,
     *               which was sent with the XML request.
     * 
     * @return <b>true</b>, if the listener claims the responsibility
     *         for the incoming message.
     *         <br><b>false</b>, otherwise
     * 
     * @see InvokeProgramData
     * @see UPES
     */
    public boolean IsResponsibleFor(InvokeProgramData msg);   
    /**
     * <b>ProcessMessage</b> will be called when the listener has
     * claimed the responsibility for a message.
     * 
     * @param u      is a reference to the UPES which called the
     *               listener for processing the message.
     *               It can be used to put a response within the
     *               same transaction to a target queue.
     * 
     * @param MQH    is the MQHandler, with which the reguest was gotten.
     * 
     * @param msg    is the request.
     * 
     * @see UPES
     * @see InvokeProgramData
     * @see MQHandler
     */
    public void    ProcessMessage(UPES u,MQHandler MQH, InvokeProgramData msg);
}
