// *****************************************************************************
// * IBM MQSeries Workflow XML Sample Scenario                                 *
// *****************************************************************************
//
//  Licensed Materials - Property of IBM
//  5697-FM3 5655-A96
//  (C) Copyright IBM Corp. 1999-2000
//  US Government Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract
//  with IBM Corp.
//
// *****************************************************************************
// *****************************************************************************
//  DISCLAIMER
//  ----------
//  This material contains programming source code for your consideration.
//  These examples have not been thoroughly tested under all conditions.
//  IBM, therefore, cannot guarantee or imply reliability, serviceability,
//  or function of these programs.
//  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
//  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
//  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
//  FOR A PARTICULAR PURPOSE.
// *****************************************************************************
package com.ibm.workflow.sample.xml;

import com.ibm.workflow.upes.MQHandler;
import com.ibm.workflow.upes.Tracer;
import com.ibm.mq.*;            // Include the MQSeries classes for Java package
import java.lang.Integer;
import java.lang.Runnable;
import java.lang.Thread;
import sun.io.ByteToCharUnicode;

/************************************************************
 * The <b>CreditRequest</b> class represents a client application
 * which can execute credit approval processes.
 * It contains member fields describing the first, last name and
 * address of the customer for who the credit approval is requested.
 * Furthermore it contains the credit amount.
 * With <b>generateExecuteRequest</b> an XML message is contructed 
 * in which the given information is put in. It is used by <b>processCreditRequest</b>
 * to process a request.
 * This class provides a log for logging certain steps of the processing.
 * Especially for logging the outgoing and incoming messages.
 *
 * This application gets its input from the 'CreditRequestClient' queue which is managed
 * by the same queue mangager as the XML input queue of the execution manager to which
 * the request is sent to. (This is only for the ease of the example).
 *
 *
 * @version 1.0
 *
 ***********************************************************/
public class CreditRequest implements Runnable
{
    private String _FirstName;
    private String _LastName;
    private String _Address;
    private int _PID;
    private int _CreditAmount;
    private Thread _t;
    private String _Log;
    /*****************************************************************************
     * <b>_QueueManagerName</b> is the queue manager which manages the MQSeries Workflow 
     * queues and the queue of this application.
     *****************************************************************************/
    private String _QueueManagerName = "FMCQM";
    /*****************************************************************************
     * <b>_Send2QName</b> is the queue name of the MQSeries Workflow execution server 
     * XML input queue.
     *****************************************************************************/
    private String _Send2QName = "EXEXMLINPUTQ";
    /*****************************************************************************
     * <b>_ReceiveFromQName</b> is the queue name from which this application 
     * receives its message input.
     *****************************************************************************/
    private String _ReceiveFromQName = "CreditRequestClient";
    private String fieldLog = new String();
    protected transient java.beans.PropertyChangeSupport propertyChange;
    /*************************************************************************************
     * <b>CreditRequest</b> is the empty contructor of the class.
     *************************************************************************************/
    public CreditRequest()
    {
    }
    private void CleanupReceiverQueue()
    {
        Tracer.trace("CreditRequest.CleanupReceiverQueue entry");
        try
        {
            while(true)
            {
                MQHandler MQH = new MQHandler(_QueueManagerName);
                MQMessage retrievedMessage = MQH.getMessageNoWait(_QueueManagerName,_ReceiveFromQName);
                Tracer.trace(retrievedMessage.toString());
                MQH.commit();
            }
        }
        catch(MQException ex)
        {
            //2033 means no msg available ... everything OK
            if(ex.reasonCode != 2033)
                Tracer.trace("An MQSeries error occurred : Completion code " +
                             ex.completionCode + " Reason code " + ex.reasonCode);
        }
        Tracer.trace("CreditRequest.CleanupReceiverQueue exit");
    }
    public synchronized void addPropertyChangeListener(java.beans.PropertyChangeListener listener)
    {
        getPropertyChange().addPropertyChangeListener(listener);
    }
    /*****************************************************************************
     * <b>appendLog</b> appends a log entry to the end of the log.
     * @parameters <b>append</b> the entry which is to be appended to the end
     * 
     *****************************************************************************/
    public void appendLog(String appendString)
    {
        setLog(getLog()+appendString+"\n");
    }
    public void firePropertyChange(String propertyName, Object oldValue, Object newValue)
    {
        getPropertyChange().firePropertyChange(propertyName, oldValue, newValue);
    }
    /*****************************************************************************
     * <b>generateExecuteRequest</b> returns a string representing the XML representation
     * of an execute process call of the MQSeries Workflow system.
     * @return a string representing the XML representation
     * of an execute process call of the MQSeries Workflow system.
     * 
     *****************************************************************************/
    public String generateExecuteRequest() 
    {
        String executeRequestMessage;
        executeRequestMessage =
            "<?xml version=\"1.0\" standalone=\"yes\"?>"
            +"\n<WfMessage>"
            +"\n  <ProcessTemplateExecute>"
            +"\n    <ProcTemplName>XML CreditRequest</ProcTemplName>"
            +"\n    <ProcInstInputData>"
            +"\n      <CreditInfo>"
            +"\n        <CreditRequestor>"
            +"\n          <FirstName>" + _FirstName.trim()+"</FirstName>"
            +"\n          <LastName>" + _LastName.trim()+"</LastName>"
            +"\n        </CreditRequestor>"
            +"\n        <Address>" + _Address.trim()+"</Address>"
            +"\n        <CreditAmount>" + Long.toString(_CreditAmount)+"</CreditAmount>"
            +"\n      </CreditInfo>"
            +"\n    </ProcInstInputData>"
            +"\n  </ProcessTemplateExecute>"
            +"\n</WfMessage>\n\n\n";

        return executeRequestMessage; 

    }
    public String getAddress()
    {
        return _Address;
    }
    public int getCreditAmount()
    {
        return _CreditAmount;
    }
    public String getFirstName()
    {
        return _FirstName;
    }
    public String getLastName()
    {
        return _LastName;
    }
    public String getLog()
    {
        return fieldLog;
    }
    public int getPID()
    {
        return _PID;
    }
    protected java.beans.PropertyChangeSupport getPropertyChange()
    {
        if(propertyChange == null)
        {
            propertyChange = new java.beans.PropertyChangeSupport(this);
        };
        return propertyChange;
    }
    public String getQueueManagerName()
    {
        return _QueueManagerName;
    }
    public String getReceiveFromQName()
    {
        return _ReceiveFromQName;
    }
    public String getSend2QName()
    {
        return _Send2QName;
    }
    public static void main(String[] args)
    {
    }
    /*****************************************************************************
    * <b>processCreditRequest</b> is the method to process an execute process 
    * of MQSeries Workflow for the credit request process.
    * First the message is constructed by the <b>generateExecuteRequest</b> method.
    * Then a connection to the queuing system is established and the message is sent.
    * Afterwards it waits for a response with the <b>WaitForResponse</b> method which
    * opens an own thread for it.
    *
    * @see WaitForResponse
    * @see generateExecuteRequest
    * 
    *****************************************************************************/
    public void processCreditRequest()
    {
        appendLog("Processing CreditRequest...");
        String executeXMLRequest = generateExecuteRequest();
        appendLog(executeXMLRequest);
        Tracer.trace("The request is: " + executeXMLRequest);

        CleanupReceiverQueue();
        try
        {

            // Create a connection to the queue manager
            MQQueueManager qMgr = new MQQueueManager(_QueueManagerName);

            // Set up the options on the queue we wish to open...
            // Note. All MQSeries Options are prefixed with MQC in Java.
            int openOptions = MQC.MQOO_INPUT_AS_Q_DEF |
                              MQC.MQOO_OUTPUT ;

            // Now specify the queue that we wish to open,
            // and the open options...
            MQQueue putQueue = qMgr.accessQueue(_Send2QName,
                                                openOptions);
            //                       null,           // default q manager
            //                       null,           // no dynamic q name
            //                       null);          // no alternate user id

            // prepare the data to put
            MQMessage msg2put = new MQMessage();
            msg2put.replyToQueueName = getReceiveFromQName();
            msg2put.replyToQueueManagerName = getQueueManagerName();
            msg2put.writeChars(executeXMLRequest);

            // specify the message options...
            MQPutMessageOptions pmo = new MQPutMessageOptions(); // accept the // defaults,
            // same as MQPMO_DEFAULT
            // put the message on the queue
            putQueue.put(msg2put,
                         pmo);

            // Close the queue...
            putQueue.close();
            WaitForResponse();

        }
        // If an error has occurred in the above, try to identify what went wrong
        // Was it an MQSeries error?

        catch(MQException ex)
        {
            Tracer.trace("An MQSeries error occurred : Completion code " +
                         ex.completionCode + " Reason code " + ex.reasonCode);
        }
        // Was it a Java buffer space error?
        catch(java.io.IOException ex)
        {
            Tracer.trace("An error occurred whilst writing to the message buffer: " + ex);
        }
    }
    public synchronized void removePropertyChangeListener(java.beans.PropertyChangeListener listener)
    {
        getPropertyChange().removePropertyChangeListener(listener);
    }

    /*****************************************************************************
    * <b>run</b> is body of threads which wait for a message to be put
    * a queue given by <b>_QueueManagerName</b> and <b>_ReceiveFromQName</b>
    * @see WaitForResponse
    * @see generateExecuteRequest
    * 
    *****************************************************************************/


    public void run()
    {
        try
        {
            MQHandler MQH = new MQHandler(_QueueManagerName);
            MQMessage retrievedMessage = MQH.getMessage(_QueueManagerName,_ReceiveFromQName);
            MQH.commit();

            appendLog("Waiting for response message...");
            // And prove we have the message by displaying the UTF message text
            byte b[] = new byte[retrievedMessage.getDataLength( ) ];
            retrievedMessage.readFully( b, 0, retrievedMessage.getDataLength( ) );
            String msgText = new String( b);

            Tracer.trace("The message is: " + msgText);
            appendLog(msgText);

        }
        catch(MQException ex)
        {
            Tracer.trace("An MQSeries error occurred : Completion code " +
                         ex.completionCode + " Reason code " + ex.reasonCode);
        }
        // Was it a Java buffer space error?
        catch(java.io.IOException ex)
        {
            Tracer.trace("An error occurred whilst writing to the message buffer: " + ex);
        }
    }
    public void setAddress(String new_Address)
    {
        _Address = new_Address;
    }
    public void setCreditAmount(int newCreditAmount)
    {
        _CreditAmount = newCreditAmount;
    }
    public void setCreditAmount(String amountAsString)
    {
        /* Die Methode setCreditAmount ausfhren. */
        setCreditAmount(Integer.parseInt(amountAsString));
        return;
    }
    public void setFirstName(String newName)
    {
        _FirstName = newName;
    }
    public void setLastName(String new_lastName)
    {
        _LastName = new_lastName;
    }
    public void setLog(String log)
    {
        String oldValue = fieldLog;
        fieldLog = log;
        firePropertyChange("log", oldValue, log);
    }
    public void setPID(int new_PID)
    {
        _PID = new_PID;
    }
    public void setQueueManagerName(String newQueueManagerName)
    {
        _QueueManagerName = newQueueManagerName;
    }
    public void setReceiveFromQName(String newReceiveFromQName)
    {
        _ReceiveFromQName = newReceiveFromQName;
    }
    public void setSend2QName(String newSend2QName)
    {
        _Send2QName = newSend2QName;
    }
    /*****************************************************************************
    * <b>WaitForResponse</b> This method creates a new thread which waits for a 
    * message.
    * 
    *****************************************************************************/
    public void WaitForResponse()
    {
        _t = new Thread(this);
        _t.start();
    }  
}

