// *****************************************************************************
// * IBM MQSeries Workflow XML Sample Scenario                                 *
// *****************************************************************************
//
//  Licensed Materials - Property of IBM
//  5697-FM3 5655-A96
//  (C) Copyright IBM Corp. 1999-2000
//  US Government Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract
//  with IBM Corp.
//
// *****************************************************************************
// *****************************************************************************
//  DISCLAIMER
//  ----------
//  This material contains programming source code for your consideration.
//  These examples have not been thoroughly tested under all conditions.
//  IBM, therefore, cannot guarantee or imply reliability, serviceability,
//  or function of these programs.
//  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
//  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
//  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
//  FOR A PARTICULAR PURPOSE.
// *****************************************************************************
package com.ibm.workflow.sample.xml;

import com.ibm.workflow.upes.*;

import java.util.Date;

/**
 * The <b>SchufaSampleListener</b> class is a sample implemenation of the UPESListener
 * interface. It accepts InvokeProgram messages when the program name is 'AssessCreditRisk'.
 * It simply returns an 'Y' in the 'AddApproval' field
 * and an 'L' in the RiskFactor field of the return output container.
 * @version 1.0
 */
class SchufaSampleListener implements UPESListener
   {
   protected transient java.util.Vector aChangeListener = null;
   protected transient java.beans.PropertyChangeSupport propertyChange;
   private javax.swing.DefaultListModel fieldLogListing = new javax.swing.DefaultListModel();

/************************************************************
 * Constructor only makes an log entry indicating that the Listener has been started.
 * 
 ***********************************************************/
   public SchufaSampleListener()
      {
      //setLogentries("");
      AppendLogEntry("SchufaSampleListener started ");
      }  
   public synchronized void addPropertyChangeListener(java.beans.PropertyChangeListener listener)
      {
      getPropertyChange().addPropertyChangeListener(listener);
      }  
/************************************************************
 * <b>AppendLogEntry</b> appends an entry to the objects log.
 * @parameters <b>append</b> is the string which is to be appended to the log.
 * 
 ***********************************************************/
   public void AppendLogEntry(String append)
      {
      getLogListing().addElement((new Date()).toString()+"   "+append);
      firePropertyChange("logListing", getLogListing(), getLogListing());
      }  
   public void firePropertyChange(String propertyName, Object oldValue, Object newValue)
      {
      getPropertyChange().firePropertyChange(propertyName, oldValue, newValue);
      }  
   public javax.swing.DefaultListModel getLogListing()
      {
      return fieldLogListing;
      }  
/*****************************************************************************
 * <b>getName</b> returns the Name of the listener and is hardcoded.
 * @return  returns "SchufaSampleListener" the Name of the listener and is hardcoded.
 * 
 *****************************************************************************/
   public String getName()
      {
      return "SchufaSampleListener";
      }  
   protected java.beans.PropertyChangeSupport getPropertyChange()
      {
      if (propertyChange == null)
         {
         propertyChange = new java.beans.PropertyChangeSupport(this);
         };
      return propertyChange;
      }  
/*****************************************************************************
 * <b>IsActive</b> always returns true because request are processed immediately.
 * 
 *****************************************************************************/

   public boolean IsActive()
      {
      AppendLogEntry("Activity signaled");
      return true;
      }  
/*****************************************************************************
 * <b>IsResponsibleFor</b> returns whether the listener is responsible for a given message
 * It is responsible for message in which the program name is either 'RequestCreditApproval'
 * or 'CreditApproval'.
 *
 * @parameters  <b>msg</b> the message for which the listener is ask for responsibility.
 * @return  <b>true</b>, if the program name of msg is 'AssessCreditRisk'
 *<p><b>false</b> otherwise
 * 
 *****************************************************************************/
   public boolean IsResponsibleFor(InvokeProgramData msg)
      {
      if (msg._ProgramName.equals("AssessCreditRisk"))
         {
         AppendLogEntry("Responsibility accepted");
         return true;
         }
      else
         {
         AppendLogEntry("Responsibility denied");
         return false;
         }
      }  
/*****************************************************************************
 * <b>ProcessMessage</b> processes the handed over message. This implementation
 * is quite simple; it always returns the same response data.
 * It simply returns an 'Y' in the 'AddApproval' field 
 * and an 'L' in the RiskFactor field of the return output container.
 * The response is immediately sent back without further interaction.
 *
 * @parameters <b>u</b> the UPES from which the call came in.
 * @parameters <b>msg</b> message which is to be processed.
 *
 * @see UPES
 * @see InvokeProgramData
 *
 *****************************************************************************/

   public void ProcessMessage(UPES u, MQHandler MQH, InvokeProgramData msg)
      {

      AppendLogEntry("Start Processing Message");
      AppendLogEntry("    Starter is: "+msg._Starter);
      Container OutputContainer = msg._ProgramInputData;
      ContainerElement RF;
      RF = OutputContainer.GetRootElement().GetChildNamed("CreditInfo").GetChildNamed("RiskFactor");
      if (RF == null)
         {
         RF = new ContainerElement("RiskFactor");
         OutputContainer.GetRootElement().GetChildNamed("CreditInfo").AddChild(RF);
         }
      RF.SetValue("L");
      ContainerElement ACT;
      ACT = OutputContainer.GetRootElement().GetChildNamed("CreditInfo").GetChildNamed("_ACTIVITY");
         if (ACT != null)
            {
            OutputContainer.GetRootElement().GetChildNamed("CreditInfo").RemoveChild(ACT);
            }
//       ACT = OutputContainer.GetRootElement().GetChildNamed("CreditInfo").GetChildNamed("_ACTIVITY");
//       if (ACT == null)
//          {
//          ACT = new ContainerElement("_ACTIVITY");
//          OutputContainer.GetRootElement().GetChildNamed("CreditInfo").AddChild(ACT);
//          }
//       ACT.SetValue("");
      ContainerElement AA;
      AA = OutputContainer.GetRootElement().GetChildNamed("CreditInfo").GetChildNamed("AddApproval");
      if (AA == null)
         {
         AA = new ContainerElement("AddApproval");
         OutputContainer.GetRootElement().GetChildNamed("CreditInfo").AddChild(AA);
         }
      AA.SetValue("Y");

      InvokeProgramResponseData responseData 
      = new InvokeProgramResponseData(msg._ActImplCorrelID,"0",OutputContainer);
      u.SendMessage(MQH,responseData,msg._ResponseQueueManagerName,msg._ResponseQueueName);
      AppendLogEntry("    SCHUFA Entry checked");
      AppendLogEntry("End Processing Message");
      }  
   public void register2UPES(UPES u) 
      {
      u.RegisterUPESListener(this);
      }
   public synchronized void removePropertyChangeListener(java.beans.PropertyChangeListener listener)
      {
      getPropertyChange().removePropertyChangeListener(listener);
      }  
   public void setLogListing(javax.swing.DefaultListModel logListing)
      {
      javax.swing.DefaultListModel oldValue = fieldLogListing;
      fieldLogListing = logListing;
      firePropertyChange("logListing", oldValue, logListing);
      }  
   }      
